
from __future__ import print_function

import sys
sys.path.insert(0, '../')
import signal
import threading
import pyuv


RESPONSE = b"HTTP/1.1 200 OK\r\n" \
           "Content-Type: text/plain\r\n" \
           "Content-Length: 12\r\n" \
           "\r\n" \
           "hello world\n"


def on_client_shutdown(client, error):
    client.close()
    clients.remove(client)

def on_read(client, data, error):
    if data is None:
        client.close()
        clients.remove(client)
        return
    data = data.strip()
    if not data:
        return
    client.write(RESPONSE)
    client.shutdown(on_client_shutdown)

def on_connection(server, error):
    client = pyuv.TCP(server.loop)
    server.accept(client)
    clients.append(client)
    client.start_read(on_read)

def async_exit(async):
    [c.close() for c in clients]
    async.close()
    signal_h.close()
    server.close()

def signal_cb(handle, signum):
    global async
    async.send()


print("PyUV version %s" % pyuv.__version__)

loop = pyuv.Loop.default_loop()

async = pyuv.Async(loop, async_exit)
clients = []

server = pyuv.TCP(loop)
server.bind(("0.0.0.0", 1234))
server.listen(on_connection)

signal_h = pyuv.Signal(loop)
signal_h.start(signal_cb, signal.SIGINT)

loop.run()

print("Stopped!")






from common import unittest2
from threading import Thread

import pyuv

import errno
import select

try:
    poller = select.epoll
    poller_type = 'epoll'
except AttributeError:
    try:
        poller = select.kqueue
        poller_type = 'kqueue'
    except AttributeError:
        poller = None


class EmbedTest(unittest2.TestCase):

    def embed_cb(self, handle):
        self.loop.run(pyuv.UV_RUN_ONCE)
        self.sem.post()

    def timer_cb(self, handle):
        self.embed_timer_called += 1
        self.embed_closed = True
        self.embed_async.close()
        handle.close()

    def poll(self, poll_obj, timeout):
        if poller_type == 'kqueue':
            poll_obj.control(None, 0, timeout)
        elif poller_type == 'epoll':
            poll_obj.poll(timeout)
        else:
            self.fail('Bogus poller type')

    def embed_runner(self):
        fd = self.loop.fileno()
        poll = poller.fromfd(fd)
        while not self.embed_closed:
            timeout = self.loop.get_timeout()
            while True:
                try:
                    self.poll(poll, timeout)
                except OSError as e:
                    if e.args[0] == errno.EINTR:
                        continue
                break
            self.embed_async.send()
            self.sem.wait()

    def test_embed(self):
        if poller is None:
            self.skipTest("test disabled if no suitable poller method is found")
            return
        self.embed_timer_called = 0
        self.embed_closed = False
        self.external = pyuv.Loop()
        self.embed_async = pyuv.Async(self.external, self.embed_cb)

        self.loop = pyuv.Loop()
        timer = pyuv.Timer(self.loop)
        timer.start(self.timer_cb, 0.25, 0)

        self.sem = pyuv.thread.Semaphore(0)
        t = Thread(target=self.embed_runner)
        t.start()
        self.external.run()
        t.join()
        external = None

        self.assertEqual(self.embed_timer_called, 1)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





from __future__ import print_function

import signal
import pyuv


def on_read(handle, ip_port, flags, data, error):
    if data is not None:
        handle.send(ip_port, data)

def signal_cb(handle, signum):
    signal_h.close()
    server.close()


print("PyUV version %s" % pyuv.__version__)

loop = pyuv.Loop.default_loop()

server = pyuv.UDP(loop)
server.bind(("0.0.0.0", 1234))
server.start_recv(on_read)

signal_h = pyuv.Signal(loop)
signal_h.start(signal_cb, signal.SIGINT)

loop.run()

print("Stopped!")





import ast
import errno
import os
import platform
import shutil
import subprocess
import sys

from distutils import log
from distutils.command.build_ext import build_ext
from distutils.command.sdist import sdist
from distutils.errors import DistutilsError


def makedirs(path):
    try:
        os.makedirs(path)
    except OSError as e:
        if e.errno==errno.EEXIST and os.path.isdir(path) and os.access(path, os.R_OK | os.W_OK | os.X_OK):
            return
        raise

def rmtree(path):
    try:
        shutil.rmtree(path)
    except OSError as e:
        if e.errno != errno.ENOENT:
            raise

def exec_process(cmdline, silent=True, input=None, **kwargs):
    """Execute a subprocess and returns the returncode, stdout buffer and stderr buffer.
    Optionally prints stdout and stderr while running."""
    try:
        sub = subprocess.Popen(args=cmdline, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)
        stdout, stderr = sub.communicate(input=input)
        returncode = sub.returncode
        if not silent:
            sys.stdout.write(stdout)
            sys.stderr.write(stderr)
    except OSError as e:
        if e.errno == errno.ENOENT:
            raise DistutilsError('"%s" is not present on this system' % cmdline[0])
        else:
            raise
    if returncode != 0:
        raise DistutilsError('Got return value %d while executing "%s", stderr output was:\n%s' % (returncode, " ".join(cmdline), stderr.decode("utf-8").rstrip("\n")))
    return stdout


def prepare_windows_env(env):
    env.pop('VS110COMNTOOLS', None)
    if sys.version_info < (3, 3):
        env.pop('VS100COMNTOOLS', None)

    if env.get('PYTHON'):
        return  # Already manually set by user.

    if (2, 6) <= sys.version_info[:2] <= (2, 7):
        env['PYTHON'] = sys.executable
        return  # The current executable is fine.

    # Try if `python` on PATH is the right one. If we would execute
    # `python` directly the current executable might be used so we
    # delegate this to cmd.
    cmd = ['cmd.exe', '/C',  'python', '-c', 'import sys; '
           'v = str(sys.version_info[:2]); sys.stdout.write(v); '
           'sys.stdout.flush()']
    try:
        sub = subprocess.Popen(cmd, stdout=subprocess.PIPE)
        stdout, _ = sub.communicate()
        version = ast.literal_eval(stdout.decode('utf-8').strip())
        if (2, 6) <= version <= (2, 7):
            return  # Python on PATH is fine
    except OSError:
        pass

    # Check default install locations
    for v in ('26', '27'):
        path = os.path.join('%SYSTEMDRIVE%', 'Python%s' % v, 'python.exe')
        path = os.path.expandvars(path)
        if os.path.isfile(path):
            log.info('Using "%s" to build libuv...' % path)
            env['PYTHON'] = path
            return

    raise DistutilsError('No appropriate Python version found. An '
                         'installation of 2.6 or 2.7 is required to '
                         'build libuv. You can set the environment '
                         'variable "PYTHON" to point to a custom '
                         'installation location.')


class libuv_build_ext(build_ext):
    libuv_dir      = os.path.join('deps', 'libuv')
    libuv_repo     = 'https://github.com/joyent/libuv.git'
    libuv_branch   = 'v0.10'
    libuv_revision = '2d72854'
    libuv_patches  = []

    user_options = build_ext.user_options
    user_options.extend([
        ("libuv-clean-compile", None, "Clean libuv tree before compilation"),
        ("libuv-force-fetch", None, "Remove libuv (if present) and fetch it again")
    ])
    boolean_options = build_ext.boolean_options
    boolean_options.extend(["libuv-clean-compile", "libuv-force-fetch"])

    def initialize_options(self):
        build_ext.initialize_options(self)
        self.libuv_clean_compile = 0
        self.libuv_force_fetch = 0

    def build_extensions(self):
        if self.compiler.compiler_type == 'mingw32':
            # Dirty hack to avoid linking with more than one C runtime when using MinGW
            # Note that this hack forces the compilation to use the old MSVCT
            self.compiler.dll_libraries = [lib for lib in self.compiler.dll_libraries if not lib.startswith('msvcr')]
        self.force = self.libuv_force_fetch or self.libuv_clean_compile
        if self.compiler.compiler_type == 'msvc':
            self.libuv_lib = os.path.join(self.libuv_dir, 'Release', 'lib', 'libuv.lib')
        else:
            self.libuv_lib = os.path.join(self.libuv_dir, 'libuv.a')
        self.get_libuv()
        # Set compiler options
        if self.compiler.compiler_type == 'mingw32':
            self.compiler.add_library_dir(self.libuv_dir)
            self.compiler.add_library('uv')
        else:
            self.extensions[0].extra_objects.extend([self.libuv_lib])
        self.compiler.add_include_dir(os.path.join(self.libuv_dir, 'include'))
        if sys.platform.startswith('linux'):
            self.compiler.add_library('rt')
        elif sys.platform == 'darwin':
            self.extensions[0].extra_link_args.extend(['-framework', 'CoreServices'])
        elif sys.platform == 'win32':
            self.extensions[0].define_macros.append(('WIN32', 1))
            if self.compiler.compiler_type == 'msvc':
                self.extensions[0].extra_link_args.extend(['/NODEFAULTLIB:libcmt', '/LTCG'])
                self.compiler.add_library('advapi32')
            self.compiler.add_library('iphlpapi')
            self.compiler.add_library('psapi')
            self.compiler.add_library('ws2_32')
        build_ext.build_extensions(self)

    def get_libuv(self):
        #self.debug_mode =  bool(self.debug) or hasattr(sys, 'gettotalrefcount')
        win32_msvc = self.compiler.compiler_type=='msvc'
        def download_libuv():
            log.info('Downloading libuv...')
            makedirs(self.libuv_dir)
            exec_process(['git', 'clone', '-b', self.libuv_branch, self.libuv_repo, self.libuv_dir])
            exec_process(['git', 'reset', '--hard', self.libuv_revision], cwd=self.libuv_dir)
        def patch_libuv():
            if self.libuv_patches:
                log.info('Patching libuv...')
                for patch_file in self.libuv_patches:
                    exec_process(['patch', '--forward', '-d', self.libuv_dir, '-p0', '-i', os.path.abspath(patch_file)])
        def build_libuv():
            cflags = '-fPIC'
            env = os.environ.copy()
            env['CFLAGS'] = ' '.join(x for x in (cflags, env.get('CFLAGS', None), env.get('ARCHFLAGS', None)) if x)
            log.info('Building libuv...')
            if win32_msvc:
                prepare_windows_env(env)
                libuv_arch = {'32bit': 'x86', '64bit': 'x64'}[platform.architecture()[0]]
                exec_process(['cmd.exe', '/C', 'vcbuild.bat', libuv_arch, 'release'], cwd=self.libuv_dir, env=env, shell=True)
            else:
                exec_process(['make', 'libuv.a'], cwd=self.libuv_dir, env=env)
        if self.libuv_force_fetch:
            rmtree('deps')
        if not os.path.exists(self.libuv_dir):
            try:
                download_libuv()
            except BaseException:
                rmtree('deps')
                raise
            patch_libuv()
            build_libuv()
        else:
            if self.libuv_clean_compile:
                if win32_msvc:
                    env = os.environ.copy()
                    prepare_windows_env(env)
                    exec_process(['cmd.exe', '/C', 'vcbuild.bat', 'clean'], cwd=self.libuv_dir, env=env, shell=True)
                    rmtree(os.path.join(self.libuv_dir, 'Release'))
                else:
                    exec_process(['make', 'clean'], cwd=self.libuv_dir)
            if not os.path.exists(self.libuv_lib):
                log.info('libuv needs to be compiled.')
                build_libuv()
            else:
                log.info('No need to build libuv.')


class libuv_sdist(sdist):
    libuv_dir      = os.path.join('deps', 'libuv')
    libuv_repo     = libuv_build_ext.libuv_repo
    libuv_branch   = libuv_build_ext.libuv_branch
    libuv_revision = libuv_build_ext.libuv_revision
    libuv_patches  = libuv_build_ext.libuv_patches

    def initialize_options(self):
        sdist.initialize_options(self)
        rmtree('deps')
        makedirs(self.libuv_dir)
        log.info('Downloading libuv...')
        exec_process(['git', 'clone', '-b', self.libuv_branch, self.libuv_repo, self.libuv_dir])
        exec_process(['git', 'checkout', self.libuv_revision], cwd=self.libuv_dir)
        if self.libuv_patches:
            log.info('Patching libuv...')
            for patch_file in self.libuv_patches:
                exec_process(['patch', '--forward', '-d', self.libuv_dir, '-p0', '-i', os.path.abspath(patch_file)])
        rmtree(os.path.join(self.libuv_dir, '.git'))





from common import unittest2
import pyuv


class IdleTest(unittest2.TestCase):

    def test_idle1(self):
        self.idle_cb_called = 0
        def idle_cb(idle):
            self.idle_cb_called += 1
            idle.stop()
            idle.close()
        loop = pyuv.Loop.default_loop()
        idle = pyuv.Idle(loop)
        idle.start(idle_cb)
        loop.run()
        self.assertEqual(self.idle_cb_called, 1)


if __name__ == '__main__':
    unittest2.main(verbosity=2)




# coding=utf8

import os
import socket
import sys

from common import unittest2
import common
import pyuv

try:
    memoryview
except NameError:
    # Fix for Python 2.6
    memoryview = str


TEST_PORT = 1234

class TCPErrorTest(unittest2.TestCase):

    def on_client_connect_error(self, client, error):
        self.assertNotEqual(error, None)
        client.close()

    def test_client1(self):
        loop = pyuv.Loop.default_loop()
        client = pyuv.TCP(loop)
        client.connect(("127.0.0.1", TEST_PORT), self.on_client_connect_error)
        loop.run()

    def test_client2(self):
        loop = pyuv.Loop.default_loop()
        client = pyuv.TCP(loop)
        self.assertFalse(client.readable)
        self.assertFalse(client.writable)
        client.close()
        loop.run()

    def test_open(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        loop = pyuv.Loop.default_loop()
        client = pyuv.TCP(loop)
        client.open(sock.fileno())
        client.connect(("127.0.0.1", TEST_PORT), self.on_client_connect_error)
        loop.run()
        sock.close()

    def test_raise(self):
        loop = pyuv.Loop.default_loop()
        tcp = pyuv.TCP(loop)
        self.assertRaises(pyuv.error.TCPError, tcp.write, b"PING")
        tcp.close()
        loop.run()


class TCPTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        self.assertEqual(error, None)
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.write(b"PING"+common.linesep)

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close()
            self.client_connections.remove(client)
            self.server.close()
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)
        self.assertTrue(client.readable)
        self.assertTrue(client.writable)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        self.assertEqual(data, b"PING"+common.linesep)
        client.close()

    def test_tcp1(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()


class TCPTest2(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.write_cb_count = 0

    def on_connection(self, server, error):
        self.assertEqual(error, None)
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        for x in range(1024):
            client.write(b"PING"*1000, self.on_client_write)
        client.close()
        server.close()

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        self.assertTrue(client.readable)
        self.assertTrue(client.writable)

    def on_client_write(self, handle, error):
        self.write_cb_count += 1

    def test_tcp_write_cancel(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()
        self.assertEqual(self.write_cb_count, 1024)


class TCPTest3(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None

    def on_connection(self, server, error):
        self.assertEqual(error, None)
        connection = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(connection)
        while connection.write_queue_size == 0:
            connection.write(b"PING"*1000)
        connection.close(self.on_connection_close)

    def on_connection_close(self, connection):
        self.client.close()
        self.server.close()

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        self.assertTrue(client.readable)
        self.assertTrue(client.writable)

    def test_tcp_write_saturate(self):
        if 'TRAVIS' in os.environ:
            self.skipTest("Test disabled on Travis")
            return
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()


class TCPTestUnicode(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None

    def on_connection(self, server, error):
        self.assertEqual(error, None)
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        if sys.version_info >= (3, 0):
            data = 'PÏNG'
            exc_type = TypeError
        else:
            data = unicode('PÏNG', 'utf-8')
            exc_type = UnicodeEncodeError
        self.assertRaises(exc_type, client.write, data)
        client.close()
        self.server.close()

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        client.close()

    def test_tcp_unicode(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()


class TCPTestMemoryview(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        self.assertEqual(error, None)
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        data = memoryview(b"PING")
        client.write(data)

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close()
            self.client_connections.remove(client)
            self.server.close()
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        self.assertEqual(data, b"PING")
        client.close()

    def test_tcp_memoryview(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()


class TCPTestNull(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        self.assertEqual(error, None)
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.write(b"PIN\x00G"+common.linesep)

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close()
            self.client_connections.remove(client)
            self.server.close()
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        self.assertEqual(data, b"PIN\x00G"+common.linesep)
        client.close()

    def test_tcp_null(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()


class TCPTestList(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.writelines([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING6", b"PING7", b"PING8", b"PING9", b"PING10", b"PING11", b"PING12"])

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close()
            self.client_connections.remove(client)
            self.server.close()
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        self.assertEqual(data, b"PING1PING2PING3PING4PING5PING6PING7PING8PING9PING10PING11PING12")
        client.close()

    def test_tcp_list(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()


class TCPTestListUnicode(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None

    def on_connection(self, server, error):
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        if sys.version_info >= (3, 0):
            data = 'PÏNG'
            exc_type = TypeError
        else:
            data = unicode('PÏNG', 'utf-8')
            exc_type = UnicodeEncodeError
        self.assertRaises(exc_type, client.writelines, [data for x in range(100)])
        client.close()
        self.server.close()

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        client.close()

    def test_tcp_list_unicode(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()


class TCPTestListNull(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.writelines([b"PING1", b"PING2", b"PING3", b"PING4", b"PING5", b"PING\x00\xFF6"])

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close()
            self.client_connections.remove(client)
            self.server.close()
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        self.assertEqual(data, b"PING1PING2PING3PING4PING5PING\x00\xFF6")
        client.close()

    def test_tcp_list_null(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()


class TCPTestInvalidData(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)

        self.assertRaises(TypeError, client.write, 1234)
        self.assertRaises(TypeError, client.write, object())
        self.assertRaises(TypeError, client.writelines, 1234)
        self.assertRaises(TypeError, client.writelines, object())

        client.close()
        self.client_connections.remove(client)
        self.server.close()

    def on_client_connection_read(self, client, data, error):
        client.close()
        self.client_connections.remove(client)
        self.server.close()
        self.fail('Expected write to fail.' % data)
        return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        self.assertEqual(data, None)
        client.close()

    def test_invalid_data(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()


class TCPShutdownTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.write(b"PING"+common.linesep)

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close(self.on_close)
            self.client_connections.remove(client)
            self.server.close(self.on_close)
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_close(self, handle):
        self.close_cb_called += 1

    def on_client_shutdown(self, client, error):
        self.shutdown_cb_called += 1
        client.close(self.on_close)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        self.assertEqual(data, b"PING"+common.linesep)
        client.shutdown(self.on_client_shutdown)

    def test_tcp_shutdown(self):
        self.shutdown_cb_called = 0
        self.close_cb_called = 0
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.client = pyuv.TCP(self.loop)
        self.client.connect(("127.0.0.1", TEST_PORT), self.on_client_connection)
        self.loop.run()
        self.assertEqual(self.shutdown_cb_called, 1)
        self.assertEqual(self.close_cb_called, 3)


class TCPFlagsTest(unittest2.TestCase):

    def test_tcp_flags(self):
        loop = pyuv.Loop.default_loop()
        tcp = pyuv.TCP(loop)
        tcp.nodelay(True)
        tcp.keepalive(True, 60)
        tcp.simultaneous_accepts(True)
        tcp.close()
        loop.run()
        self.assertTrue(True)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





import weakref

from common import unittest2
import pyuv


class WeakrefTest(unittest2.TestCase):
    handle_types = ('Check', 'Idle', 'Pipe', 'Prepare', 'TCP', 'Timer', 'UDP')

    def test_weakref(self):
        loop = pyuv.Loop()
        refs = []
        for type in self.handle_types:
            klass = getattr(pyuv, type)
            obj = klass(loop)
            refs.append(weakref.ref(obj))
            del obj
        for ref in refs:
            self.assertNotEqual(ref(), None)
        loop.run()
        for ref in refs:
            self.assertEqual(ref(), None)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





from common import unittest2, platform_skip
from functools import partial

import pyuv
import random
import sys

TEST_PORT = random.randint(2000, 3000)
TEST_PORT2 = random.randint(2000, 3000)
TEST_ADDR = ("127.0.0.1", TEST_PORT)
TEST_ADDR2 = ("127.0.0.1", TEST_PORT2)


if sys.platform == 'win32':
    TEST_PIPE = '\\\\.\\pipe\\test-pipe'
else:
    TEST_PIPE = 'test-pipe'


class IPCTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()

    def proc_exit_cb(self, proc, exit_status, term_signal):
        proc.close()

    def on_client_connection(self, client, error):
        client.close()
        self.connections.remove(client)

    def make_many_connections(self):
        for i in range(100):
            conn = pyuv.TCP(self.loop)
            self.connections.append(conn)
            conn.connect(TEST_ADDR, self.on_client_connection)

    def on_ipc_connection(self, handle, error):
        if self.local_conn_accepted:
            return
        conn = pyuv.TCP(self.loop)
        self.tcp_server.accept(conn)
        conn.close()
        self.tcp_server.close()
        self.local_conn_accepted = True

    def on_channel_read(self, handle, data, pending, error):
        if self.tcp_server is None:
            self.assertEqual(pending, pyuv.UV_TCP)
            self.tcp_server = pyuv.TCP(self.loop)
            self.channel.accept(self.tcp_server)
            self.tcp_server.listen(self.on_ipc_connection, 12)
            self.assertEqual(data.strip(), b"hello")
            self.channel.write(b"world")
            self.make_many_connections()
        else:
            if data.strip() == b"accepted_connection":
                self.assertEqual(pending, pyuv.UV_UNKNOWN_HANDLE)
                self.channel.close()

    def _do_test(self, test_type):
        self.connections = []
        self.local_conn_accepted = False
        self.tcp_server = None
        self.channel = pyuv.Pipe(self.loop, True)
        stdio = [pyuv.StdIO(stream=self.channel, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_READABLE_PIPE|pyuv.UV_WRITABLE_PIPE)]
        proc = pyuv.Process(self.loop)
        proc.spawn(file=sys.executable, args=["proc_ipc.py", test_type, str(TEST_PORT)], exit_callback=self.proc_exit_cb, stdio=stdio)
        self.channel.start_read2(self.on_channel_read)
        self.loop.run()

    def test_ipc1(self):
        self._do_test("listen_before_write")

    def test_ipc2(self):
        self._do_test("listen_after_write")


class IPCSendRecvTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()

    def proc_exit_cb(self, proc, exit_status, term_signal):
        proc.close()

    def on_channel_read(self, expected_type, handle, data, pending, error):
        self.assertEqual(pending, expected_type)
        if pending == pyuv.UV_NAMED_PIPE:
            recv_handle = pyuv.Pipe(self.loop)
        elif pending == pyuv.UV_TCP:
            recv_handle = pyuv.TCP(self.loop)
        elif pending == pyuv.UV_UDP:
            recv_handle = pyuv.UDP(self.loop)
        self.channel.accept(recv_handle)
        self.channel.close()
        self.send_pipe.close()
        recv_handle.close()

    def _do_test(self):
        self.channel = pyuv.Pipe(self.loop, True)
        stdio = [pyuv.StdIO(stream=self.channel, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_READABLE_PIPE|pyuv.UV_WRITABLE_PIPE)]
        proc = pyuv.Process(self.loop)
        proc.spawn(file=sys.executable, args=["proc_ipc_echo.py"], exit_callback=self.proc_exit_cb, stdio=stdio)
        self.channel.write2(b".", self.send_handle)
        self.channel.start_read2(functools.partial(self.on_channel_read, self.send_handle_type))
        self.loop.run()

    @platform_skip(["win32"])
    def test_ipc_send_recv_pipe(self):
        # Handle that will be sent to the process and back
        self.send_handle = pyuv.Pipe(self.loop, True)
        self.send_handle.bind(TEST_PIPE+'2')
        self.send_handle_type = pyuv.UV_NAMED_PIPE

    def test_ipc_send_recv_tcp(self):
        # Handle that will be sent to the process and back
        self.send_handle = pyuv.TCP(self.loop)
        self.send_handle.bind(TEST_ADDR2)
        self.send_handle_type = pyuv.UV_TCP

    @platform_skip(["win32"])
    def test_ipc_send_recv_udp(self):
        # Handle that will be sent to the process and back
        self.send_handle = pyuv.UDP(self.loop)
        self.send_handle.bind(TEST_ADDR2)
        self.send_handle_type = pyuv.UV_UDP


if __name__ == '__main__':
    unittest2.main(verbosity=2)





from common import unittest2
import pyuv


class PrepareTest(unittest2.TestCase):

    def test_prepare1(self):
        self.prepare_cb_called = 0
        def prepare_cb(prepare):
            self.prepare_cb_called += 1
            prepare.stop()
            prepare.close()
        loop = pyuv.Loop.default_loop()
        prepare = pyuv.Prepare(loop)
        prepare.start(prepare_cb)
        loop.run()
        self.assertEqual(self.prepare_cb_called, 1)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





import os
try:
    import pwd
except ImportError:
    pwd = None
import sys

from common import platform_skip, unittest2
import common
import pyuv


class ProcessTest(unittest2.TestCase):

    def test_process_basic(self):
        self.exit_cb_called = 0
        self.close_cb_called = 0
        def proc_close_cb(proc):
            self.close_cb_called +=1
        def proc_exit_cb(proc, exit_status, term_signal):
            self.assertEqual(exit_status, 0)
            self.exit_cb_called += 1
            proc.close(proc_close_cb)
        loop = pyuv.Loop.default_loop()
        proc = pyuv.Process(loop)
        proc.spawn(file=sys.executable, args=["proc_basic.py"],
                   exit_callback=proc_exit_cb)
        pid = proc.pid
        loop.run()
        self.assertEqual(self.exit_cb_called, 1)
        self.assertEqual(self.close_cb_called, 1)
        self.assertNotEqual(pid, None)

    def test_process_cwd(self):
        self.exit_cb_called = 0
        self.close_cb_called = 0
        def proc_close_cb(proc):
            self.close_cb_called +=1
        def proc_exit_cb(proc, exit_status, term_signal):
            self.assertEqual(exit_status, 0)
            self.exit_cb_called += 1
            proc.close(proc_close_cb)
        loop = pyuv.Loop.default_loop()
        proc = pyuv.Process(loop)
        proc.spawn(file=sys.executable, args=["proc_basic.py"],
                   exit_callback=proc_exit_cb, cwd=".")
        loop.run()
        self.assertEqual(self.exit_cb_called, 1)
        self.assertEqual(self.close_cb_called, 1)

    def test_process_stdout(self):
        self.exit_cb_called = 0
        self.close_cb_called = 0
        self.received_output = None
        def handle_close_cb(handle):
            self.close_cb_called +=1
        def proc_exit_cb(proc, exit_status, term_signal):
            self.assertEqual(exit_status, 0)
            self.exit_cb_called += 1
            proc.close(handle_close_cb)
        def stdout_read_cb(handle, data, error):
            if data is not None:
                self.received_output = data.strip()
            handle.close(handle_close_cb)
        loop = pyuv.Loop.default_loop()
        stdout_pipe = pyuv.Pipe(loop)
        stdio = []
        stdio.append(pyuv.StdIO(flags=pyuv.UV_IGNORE))
        stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
        proc = pyuv.Process(loop)
        proc.spawn(file=sys.executable, args=["proc_stdout.py"],
                   exit_callback=proc_exit_cb, stdio=stdio)
        stdout_pipe.start_read(stdout_read_cb)
        loop.run()
        self.assertEqual(self.exit_cb_called, 1)
        self.assertEqual(self.close_cb_called, 2)
        self.assertEqual(self.received_output, b"TEST")

    def test_process_args(self):
        self.exit_cb_called = 0
        self.close_cb_called = 0
        self.received_output = None
        def handle_close_cb(handle):
            self.close_cb_called +=1
        def proc_exit_cb(proc, exit_status, term_signal):
            self.assertEqual(exit_status, 0)
            self.exit_cb_called += 1
            proc.close(handle_close_cb)
        def stdout_read_cb(handle, data, error):
            self.received_output = data.strip()
            handle.close(handle_close_cb)
        loop = pyuv.Loop.default_loop()
        stdout_pipe = pyuv.Pipe(loop)
        stdio = []
        stdio.append(pyuv.StdIO(flags=pyuv.UV_IGNORE))
        stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
        proc = pyuv.Process(loop)
        proc.spawn(file=sys.executable, args=["proc_args_stdout.py", "TEST"],
                   exit_callback=proc_exit_cb, stdio=stdio)
        stdout_pipe.start_read(stdout_read_cb)
        loop.run()
        self.assertEqual(self.exit_cb_called, 1)
        self.assertEqual(self.close_cb_called, 2)
        self.assertEqual(self.received_output, b"TEST")

    def test_process_env(self):
        self.exit_cb_called = 0
        self.close_cb_called = 0
        self.received_output = None
        def handle_close_cb(handle):
            self.close_cb_called +=1
        def proc_exit_cb(proc, exit_status, term_signal):
            self.assertEqual(exit_status, 0)
            self.exit_cb_called += 1
            proc.close(handle_close_cb)
        def stdout_read_cb(handle, data, error):
            self.received_output = data.strip()
            handle.close(handle_close_cb)
        loop = pyuv.Loop.default_loop()
        stdout_pipe = pyuv.Pipe(loop)
        stdio = []
        stdio.append(pyuv.StdIO(flags=pyuv.UV_IGNORE))
        stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
        proc = pyuv.Process(loop)
        proc.spawn(file=sys.executable, args=["proc_env_stdout.py"],
                   env={"TEST": "TEST"}, exit_callback=proc_exit_cb,
                   stdio=stdio)
        stdout_pipe.start_read(stdout_read_cb)
        loop.run()
        self.assertEqual(self.exit_cb_called, 1)
        self.assertEqual(self.close_cb_called, 2)
        self.assertEqual(self.received_output, b"TEST")

    def test_process_stdin(self):
        self.exit_cb_called = 0
        self.close_cb_called = 0
        self.received_output = None
        self.exit_status = -1
        self.term_signal = 0
        def handle_close_cb(handle):
            self.close_cb_called +=1
        def proc_exit_cb(proc, exit_status, term_signal):
            self.exit_cb_called += 1
            self.exit_status = exit_status
            self.term_signal = term_signal
            proc.close(handle_close_cb)
        def stdout_read_cb(handle, data, error):
            if data:
                self.received_output = data.strip()
            handle.close(handle_close_cb)
        def stdin_write_cb(handle, error):
            handle.close(handle_close_cb)
        loop = pyuv.Loop.default_loop()
        stdin_pipe = pyuv.Pipe(loop)
        stdout_pipe = pyuv.Pipe(loop)
        stdio = []
        stdio.append(pyuv.StdIO(stream=stdin_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_READABLE_PIPE))
        stdio.append(pyuv.StdIO(stream=stdout_pipe, flags=pyuv.UV_CREATE_PIPE|pyuv.UV_WRITABLE_PIPE))
        proc = pyuv.Process(loop)
        proc.spawn(file=sys.executable, args=["proc_stdin_stdout.py"],
                   exit_callback=proc_exit_cb, stdio=stdio)
        stdout_pipe.start_read(stdout_read_cb)
        stdin_pipe.write(b"TEST"+common.linesep, stdin_write_cb)
        loop.run()
        self.assertEqual(self.exit_cb_called, 1)
        self.assertEqual(self.close_cb_called, 3)
        self.assertEqual(self.received_output, b"TEST")

    def test_process_kill(self):
        self.exit_cb_called = 0
        self.close_cb_called = 0
        self.exit_status = -1
        self.term_signal = 0
        def handle_close_cb(proc):
            self.close_cb_called +=1
        def proc_exit_cb(proc, exit_status, term_signal):
            self.exit_cb_called += 1
            self.exit_status = exit_status
            self.term_signal = term_signal
            proc.close(handle_close_cb)
        def timer_cb(timer):
            timer.close(handle_close_cb)
            proc.kill(15)
        loop = pyuv.Loop.default_loop()
        timer = pyuv.Timer(loop)
        timer.start(timer_cb, 0.1, 0)
        proc = pyuv.Process(loop)
        proc.spawn(file=sys.executable, args=["proc_infinite.py"],
                   exit_callback=proc_exit_cb)
        loop.run()
        self.assertEqual(self.exit_cb_called, 1)
        self.assertEqual(self.close_cb_called, 2)
        if sys.platform == 'win32':
            self.assertEqual(self.exit_status, 1)
        else:
            self.assertEqual(self.exit_status, 0)
        self.assertEqual(self.term_signal, 15)

    @platform_skip(["win32"])
    def test_process_uid_gid(self):
        self.exit_cb_called = 0
        self.close_cb_called = 0
        def proc_close_cb(proc):
            self.close_cb_called +=1
        def proc_exit_cb(proc, exit_status, term_signal):
            self.assertEqual(exit_status, 0)
            self.exit_cb_called += 1
            proc.close(proc_close_cb)
        if os.getuid() != 0:
            self.skipTest("test disabled if running as non-root")
            return
        p_info = pwd.getpwnam("nobody")
        loop = pyuv.Loop.default_loop()
        proc = pyuv.Process(loop)
        proc.spawn(file="./proc_basic.py", exit_callback=proc_exit_cb, uid=p_info.pw_uid, gid=p_info.pw_gid, flags=pyuv.UV_PROCESS_SETUID|pyuv.UV_PROCESS_SETGID)
        pid = proc.pid
        loop.run()
        self.assertEqual(self.exit_cb_called, 1)
        self.assertEqual(self.close_cb_called, 1)
        self.assertNotEqual(pid, None)

    @platform_skip(["win32"])
    def test_process_uid_fail(self):
        self.exit_cb_called = 0
        self.close_cb_called = 0
        def proc_close_cb(proc):
            self.close_cb_called +=1
        def proc_exit_cb(proc, exit_status, term_signal):
            self.assertNotEqual(exit_status, 0)
            self.exit_cb_called += 1
            proc.close(proc_close_cb)
        if os.getuid() != 0:
            self.skipTest("test disabled if running as non-root")
            return
        loop = pyuv.Loop.default_loop()
        proc = pyuv.Process(loop)
        proc.spawn(file="./proc_basic.py", exit_callback=proc_exit_cb, uid=-42424242, flags=pyuv.UV_PROCESS_SETUID)
        loop.run()
        self.assertEqual(self.exit_cb_called, 1)
        self.assertEqual(self.close_cb_called, 1)

    def test_process_detached(self):
        self.exit_cb_called = 0
        def proc_exit_cb(proc, exit_status, term_signal):
            self.exit_cb_called += 1
        loop = pyuv.Loop.default_loop()
        proc = pyuv.Process(loop)
        proc.spawn(file=sys.executable, args=["proc_basic.py"],
                   exit_callback=proc_exit_cb, flags=pyuv.UV_PROCESS_DETACHED)
        proc.unref()
        pid = proc.pid
        loop.run()
        self.assertEqual(self.exit_cb_called, 0)
        proc.kill(0)
        proc.kill(15)
        self.assertNotEqual(pid, None)


if __name__ == '__main__':
    unittest2.main(verbosity=2)



import signal
import sys
import pyuv


def on_pipe_read(handle, data, error):
    if data is None or data == b"exit":
        pipe_stdin.close()
        pipe_stdout.close()
    else:
        pipe_stdout.write(data)

def signal_cb(handle, signum):
    if not pipe_stdin.closed:
        pipe_stdin.close()
    if not pipe_stdin.closed:
        pipe_stdout.close()
    signal_h.close()


loop = pyuv.Loop.default_loop()

pipe_stdin = pyuv.Pipe(loop)
pipe_stdin.open(sys.stdin.fileno())
pipe_stdin.start_read(on_pipe_read)

pipe_stdout = pyuv.Pipe(loop)
pipe_stdout.open(sys.stdout.fileno())

signal_h = pyuv.Signal(loop)
signal_h.start(signal_cb, signal.SIGINT)

loop.run()





from __future__ import print_function

import pyuv
import signal
import socket


reader, writer = socket.socketpair()
reader.setblocking(False)
writer.setblocking(False)

def prepare_cb(handle):
    print("Inside prepare_cb")

def excepthook(typ, val, tb):
    print("Inside excepthook")
    if typ is KeyboardInterrupt:
        prepare.stop()
        signal_checker.stop()

loop = pyuv.Loop.default_loop()
loop.excepthook = excepthook
prepare = pyuv.Prepare(loop)
prepare.start(prepare_cb)

signal.set_wakeup_fd(writer.fileno())
signal_checker = pyuv.util.SignalChecker(loop, reader.fileno())
signal_checker.start()

loop.run()





from common import unittest2

import pyuv
import socket


class TestBasetype(unittest2.TestCase):

    def _inheritance_test(self, base, *args, **kwargs):
        derived = type(base.__name__, (base,), {})
        assert derived is not base
        assert issubclass(derived, base)
        instance = derived(*args, **kwargs)
        assert isinstance(instance, base)
        assert isinstance(instance, derived)

    def test_inherit_loop(self):
        self._inheritance_test(pyuv.Loop)

    def test_inherit_timer(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.Timer, loop)

    def test_inherit_tcp(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.TCP, loop)

    def test_inherit_udp(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.UDP, loop)

    def test_inherit_poll(self):
        loop = pyuv.Loop.default_loop()
        sock = socket.socket()
        self._inheritance_test(pyuv.Poll, loop, sock.fileno())
        sock.close()

    def test_inherit_pipe(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.Pipe, loop)

    def test_inherit_process(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.Process, loop)

    def test_inherit_async(self):
        loop = pyuv.Loop.default_loop()
        callback = lambda handle: handle
        self._inheritance_test(pyuv.Async, loop, callback)

    def test_inherit_prepare(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.Prepare, loop)

    def test_inherit_idle(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.Idle, loop)

    def test_inherit_check(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.Check, loop)

    def test_inherit_signal(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.Signal, loop)

    def test_inherit_fs_fspoll(self):
        loop = pyuv.Loop.default_loop()
        self._inheritance_test(pyuv.fs.FSPoll, loop)

    def test_inherit_thread_barrier(self):
        self._inheritance_test(pyuv.thread.Barrier, 1)

    def test_inherit_thread_condition(self):
        self._inheritance_test(pyuv.thread.Condition)

    def test_inherit_thread_mutex(self):
        self._inheritance_test(pyuv.thread.Mutex)

    def test_inherit_thread_rwlock(self):
        self._inheritance_test(pyuv.thread.RWLock)

    def test_inherit_thread_semaphore(self):
        self._inheritance_test(pyuv.thread.Semaphore, 1)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





import os
import sys
sys.path.insert(0, '../')


if sys.version_info < (2, 7) or (0x03000000 <= sys.hexversion < 0x03010000):
    # py26 or py30
    import unittest2
else:
    import unittest as unittest2


if sys.version_info >= (3, 0):
    linesep = os.linesep.encode()
else:
    linesep = os.linesep


platform = 'linux' if sys.platform.startswith('linux') else sys.platform

# decorator for class
def platform_skip(platform_list):
    def _noop(obj):
        return obj
    if platform in platform_list:
        return unittest2.skip("Test disabled in the current platform")
    return _noop





#!/usr/bin/env python

import sys
sys.path.insert(0, '../')

import pyuv


def on_channel_write(handle, error):
    global channel, tcp_server
    channel.close()
    tcp_server.close()

def on_ipc_connection(handle, error):
    global channel, connection_accepted, loop, tcp_server
    if connection_accepted:
        return
    conn = pyuv.TCP(loop)
    tcp_server.accept(conn)
    conn.close()
    channel.write(b"accepted_connection", on_channel_write)
    connection_accepted = True


connection_accepted = False
listen_after_write = sys.argv[1] == "listen_after_write"
port = int(sys.argv[2])

loop = pyuv.Loop.default_loop()

channel = pyuv.Pipe(loop, True)
channel.open(sys.stdin.fileno())

tcp_server = pyuv.TCP(loop)
tcp_server.bind(("0.0.0.0", port))
if not listen_after_write:
    tcp_server.listen(on_ipc_connection, 12)

channel.write2(b"hello", tcp_server)

if listen_after_write:
    tcp_server.listen(on_ipc_connection, 12)

loop.run()





import os
import signal
import time
import threading

from common import unittest2, platform_skip
import pyuv


@platform_skip(["win32"])
class SignalTest(unittest2.TestCase):

    def signal_cb(self, handle, signum):
        self.assertEqual(signum, signal.SIGUSR1)
        self.signal_cb_called += 1
        self.async.send()

    def async_cb(self, async):
        self.async_cb_called += 1
        self.async.close()
        self.signal_h.close()

    def test_signal1(self):
        self.async_cb_called = 0
        self.signal_cb_called = 0
        self.loop = pyuv.Loop.default_loop()
        self.async = pyuv.Async(self.loop, self.async_cb)
        self.signal_h = pyuv.Signal(self.loop)
        self.signal_h.start(self.signal_cb, signal.SIGUSR1)
        thread = threading.Thread(target=self.loop.run)
        thread.start()
        os.kill(os.getpid(), signal.SIGUSR1)
        thread.join()
        self.assertEqual(self.async_cb_called, 1)
        self.assertEqual(self.signal_cb_called, 1)


@platform_skip(["win32"])
class MultiLoopSignalTest(unittest2.TestCase):

    def setUp(self):
        self.lock = threading.Lock()
        self.signal_cb_called = 0

    def signal_cb(self, handle, signum):
        self.assertEqual(signum, signal.SIGUSR1)
        with self.lock:
            self.signal_cb_called += 1
        handle.close()

    def run_loop(self):
        loop = pyuv.Loop()
        signal_h = pyuv.Signal(loop)
        signal_h.start(self.signal_cb, signal.SIGUSR1)
        loop.run()

    def test_multi_loop_signals(self):
        threads = [threading.Thread(target=self.run_loop) for x in range(25)]
        [t.start() for t in threads]
        # Wait until threads have started
        time.sleep(1)
        os.kill(os.getpid(), signal.SIGUSR1)
        [t.join() for t in threads]
        self.assertEqual(self.signal_cb_called, 25)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





from __future__ import print_function

import signal
import pyuv


def on_read(client, data, error):
    if data is None:
        client.close()
        clients.remove(client)
        return
    client.write(data)

def on_connection(server, error):
    client = pyuv.TCP(server.loop)
    server.accept(client)
    clients.append(client)
    client.start_read(on_read)

def signal_cb(handle, signum):
    [c.close() for c in clients]
    signal_h.close()
    server.close()


print("PyUV version %s" % pyuv.__version__)

loop = pyuv.Loop.default_loop()
clients = []

server = pyuv.TCP(loop)
server.bind(("0.0.0.0", 1234))
server.listen(on_connection)

signal_h = pyuv.Signal(loop)
signal_h.start(signal_cb, signal.SIGINT)

loop.run()
print("Stopped!")





import os
import shutil
import stat

from common import unittest2
import pyuv

# Make stat return integers
os.stat_float_times(False)


BAD_FILE = 'test_file_bad'
TEST_FILE = 'test_file_1234'
TEST_FILE2 = 'test_file_1234_2'
TEST_LINK = 'test_file_1234_link'
TEST_DIR = 'test-dir'
TEST_DIR2 = 'test-dir_2'
BAD_DIR = 'test-dir-bad'
MAX_INT32_VALUE = 2 ** 31 - 1


class FSTestStat(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        os.remove(TEST_FILE)

    def stat_cb(self, loop, path, stat_result, errorno):
        self.errorno = errorno

    def test_stat_error(self):
        self.errorno = None
        pyuv.fs.stat(self.loop, BAD_FILE, self.stat_cb)
        self.loop.run()
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)

    def test_stat(self):
        self.errorno = None
        pyuv.fs.stat(self.loop, TEST_FILE, self.stat_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)

    def test_stat_sync(self):
        self.stat_data = pyuv.fs.stat(self.loop, TEST_FILE)
        self.assertNotEqual(self.stat_data, None)

    def test_stat_error_sync(self):
        try:
            pyuv.fs.stat(self.loop, BAD_FILE)
        except pyuv.error.FSError as e:
            self.errorno = e.args[0]
        else:
            self.errorno = None
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)


class FSTestLstat(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')
        try:
            pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0)
        except pyuv.error.FSError as e:
            if e.args[0] == pyuv.errno.UV_EPERM:
                raise unittest2.SkipTest("Symlinks not permitted")

    def tearDown(self):
        os.remove(TEST_FILE)
        try:
            os.remove(TEST_LINK)
        except OSError:
            pass

    def stat_cb(self, loop, path, stat_result, errorno):
        self.errorno = errorno

    def test_lstat(self):
        self.errorno = None
        pyuv.fs.lstat(self.loop, TEST_LINK, self.stat_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)


class FSTestFstat(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        os.remove(TEST_FILE)

    def fstat_cb(self, loop, path, stat_data, errorno):
        self.errorno = errorno

    def test_fstat(self):
        self.errorno = None
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
        pyuv.fs.fstat(self.loop, fd, self.fstat_cb)
        self.loop.run()
        pyuv.fs.close(self.loop, fd)
        self.assertEqual(self.errorno, None)

    def test_fstat_sync(self):
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
        self.stat_data = pyuv.fs.fstat(self.loop, fd)
        pyuv.fs.close(self.loop, fd)
        self.assertNotEqual(self.stat_data, None)


class FSTestUnlink(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        try:
            os.remove(TEST_FILE)
        except OSError:
            pass

    def bad_unlink_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_bad_unlink(self):
        self.errorno = None
        pyuv.fs.unlink(self.loop, BAD_FILE, self.bad_unlink_cb)
        self.loop.run()
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)

    def unlink_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_unlink(self):
        self.errorno = None
        pyuv.fs.unlink(self.loop, TEST_FILE, self.unlink_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)

    def test_unlink_sync(self):
        pyuv.fs.unlink(self.loop, TEST_FILE)

    def test_unlink_error_sync(self):
        try:
            pyuv.fs.unlink(self.loop, BAD_FILE)
        except pyuv.error.FSError as e:
            self.errorno = e.args[0]
        else:
            self.errorno = None
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)


class FSTestMkdir(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        os.mkdir(BAD_DIR, 0o755)

    def tearDown(self):
        os.rmdir(BAD_DIR)
        try:
            os.rmdir(TEST_DIR)
        except OSError:
            pass

    def mkdir_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_bad_mkdir(self):
        self.errorno = None
        pyuv.fs.mkdir(self.loop, BAD_DIR, 0o755, self.mkdir_cb)
        self.loop.run()
        self.assertEqual(self.errorno, pyuv.errno.UV_EEXIST)

    def test_mkdir(self):
        self.errorno = None
        pyuv.fs.mkdir(self.loop, TEST_DIR, 0o755, self.mkdir_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertTrue(os.path.isdir(TEST_DIR))

    def test_mkdir_sync(self):
        pyuv.fs.mkdir(self.loop, TEST_DIR, 0o755)
        self.assertTrue(os.path.isdir(TEST_DIR))

    def test_mkdir_error_sync(self):
        try:
            pyuv.fs.mkdir(self.loop, BAD_DIR, 0o755)
        except pyuv.error.FSError as e:
            self.errorno = e.args[0]
        else:
            self.errorno = None
        self.assertEqual(self.errorno, pyuv.errno.UV_EEXIST)


class FSTestRmdir(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        os.mkdir(TEST_DIR, 0o755)

    def tearDown(self):
        try:
            os.rmdir(TEST_DIR)
        except OSError:
            pass

    def rmdir_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_bad_rmdir(self):
        self.errorno = None
        pyuv.fs.rmdir(self.loop, BAD_DIR, self.rmdir_cb)
        self.loop.run()
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)

    def test_rmdir(self):
        self.errorno = None
        pyuv.fs.rmdir(self.loop, TEST_DIR, self.rmdir_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertFalse(os.path.isdir(TEST_DIR))

    def test_rmdir_sync(self):
        pyuv.fs.rmdir(self.loop, TEST_DIR)
        self.assertFalse(os.path.isdir(TEST_DIR))

    def test_rmdir_error_sync(self):
        try:
            pyuv.fs.rmdir(self.loop, BAD_DIR)
        except pyuv.error.FSError as e:
            self.errorno = e.args[0]
        else:
            self.errorno = None
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)


class FSTestRename(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        try:
            os.remove(TEST_FILE)
        except OSError:
            pass
        try:
            os.remove(TEST_FILE2)
        except OSError:
            pass

    def rename_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_rename(self):
        self.errorno = None
        pyuv.fs.rename(self.loop, TEST_FILE, TEST_FILE2, self.rename_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertFalse(os.path.exists(TEST_FILE))
        self.assertTrue(os.path.exists(TEST_FILE2))

    def test_rename_sync(self):
        pyuv.fs.rename(self.loop, TEST_FILE, TEST_FILE2)
        self.assertFalse(os.path.exists(TEST_FILE))
        self.assertTrue(os.path.exists(TEST_FILE2))


class FSTestChmod(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        os.remove(TEST_FILE)

    def chmod_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_chmod(self):
        self.errorno = None
        pyuv.fs.chmod(self.loop, TEST_FILE, 0o777, self.chmod_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        mode = os.stat(TEST_FILE).st_mode
        self.assertTrue(bool(mode & stat.S_IRWXU) and bool(mode & stat.S_IRWXG) and bool(mode & stat.S_IRWXO))

    def test_chmod_sync(self):
        pyuv.fs.chmod(self.loop, TEST_FILE, 0o777)
        mode = os.stat(TEST_FILE).st_mode
        self.assertTrue(bool(mode & stat.S_IRWXU) and bool(mode & stat.S_IRWXG) and bool(mode & stat.S_IRWXO))


class FSTestFchmod(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        os.remove(TEST_FILE)

    def fchmod_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_fchmod(self):
        self.errorno = None
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD | stat.S_IWRITE)
        pyuv.fs.fchmod(self.loop, fd, 0o777, self.fchmod_cb)
        self.loop.run()
        pyuv.fs.close(self.loop, fd)
        self.assertEqual(self.errorno, None)
        mode = os.stat(TEST_FILE).st_mode
        self.assertTrue(bool(mode & stat.S_IRWXU) and bool(mode & stat.S_IRWXG) and bool(mode & stat.S_IRWXO))

    def test_fchmod_sync(self):
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD | stat.S_IWRITE)
        pyuv.fs.fchmod(self.loop, fd, 0o777)
        pyuv.fs.close(self.loop, fd)
        mode = os.stat(TEST_FILE).st_mode
        self.assertTrue(bool(mode & stat.S_IRWXU) and bool(mode & stat.S_IRWXG) and bool(mode & stat.S_IRWXO))


class FSTestLink(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        os.remove(TEST_FILE)
        os.remove(TEST_LINK)

    def link_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_link(self):
        self.errorno = None
        pyuv.fs.link(self.loop, TEST_FILE, TEST_LINK, self.link_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertEqual(os.stat(TEST_FILE).st_ino, os.stat(TEST_LINK).st_ino)

    def test_link_sync(self):
        pyuv.fs.link(self.loop, TEST_FILE, TEST_LINK)
        self.assertEqual(os.stat(TEST_FILE).st_ino, os.stat(TEST_LINK).st_ino)


class FSTestSymlink(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        os.remove(TEST_FILE)
        try:
            os.remove(TEST_LINK)
        except OSError:
            pass

    def symlink_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_symlink(self):
        self.errorno = None
        pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0, self.symlink_cb)
        self.loop.run()
        if self.errorno == pyuv.errno.UV_EPERM:
            raise unittest2.SkipTest("Symlinks not permitted")
        self.assertEqual(self.errorno, None)
        self.assertTrue(os.stat(TEST_LINK).st_mode & stat.S_IFLNK)

    def test_symlink_sync(self):
        try:
            pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0)
        except pyuv.error.FSError as e:
            if e.args[0] == pyuv.errno.UV_EPERM:
                raise unittest2.SkipTest("Symlinks not permitted")
        self.assertTrue(os.stat(TEST_LINK).st_mode & stat.S_IFLNK)


class FSTestReadlink(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')
        try:
            pyuv.fs.symlink(self.loop, TEST_FILE, TEST_LINK, 0)
        except pyuv.error.FSError as e:
            if e.args[0] == pyuv.errno.UV_EPERM:
                raise unittest2.SkipTest("Symlinks not permitted")

    def tearDown(self):
        os.remove(TEST_FILE)
        try:
            os.remove(TEST_LINK)
        except OSError:
            pass

    def readlink_cb(self, loop, path, errorno):
        self.errorno = errorno
        self.link_path = path

    def test_readlink(self):
        self.errorno = None
        self.link_path = None
        pyuv.fs.readlink(self.loop, TEST_LINK, self.readlink_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertEqual(self.link_path, TEST_FILE)

    def test_readlink_sync(self):
        self.link_path = pyuv.fs.readlink(self.loop, TEST_LINK)
        self.assertEqual(self.link_path, TEST_FILE)


class FSTestChown(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        os.remove(TEST_FILE)

    def chown_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_chown(self):
        self.errorno = None
        pyuv.fs.chown(self.loop, TEST_FILE, -1, -1, self.chown_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)

    def test_chown_sync(self):
        pyuv.fs.chown(self.loop, TEST_FILE, -1, -1)


class FSTestFchown(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test')

    def tearDown(self):
        os.remove(TEST_FILE)

    def fchown_cb(self, loop, path, errorno):
        self.errorno = errorno

    def test_fchown(self):
        self.errorno = None
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD | stat.S_IWRITE)
        pyuv.fs.fchown(self.loop, fd, -1, -1, self.fchown_cb)
        self.loop.run()
        pyuv.fs.close(self.loop, fd)
        self.assertEqual(self.errorno, None)

    def test_fchown_sync(self):
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD | stat.S_IWRITE)
        pyuv.fs.fchown(self.loop, fd, -1, -1)
        pyuv.fs.close(self.loop, fd)


class FSTestOpen(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        try:
            os.remove(TEST_FILE)
        except OSError:
            pass

    def tearDown(self):
        try:
            os.remove(TEST_FILE)
        except OSError:
            pass

    def close_cb(self, loop, path, errorno):
        self.errorno = errorno

    def open_cb(self, loop, path, fd, errorno):
        self.assertNotEqual(fd, None)
        self.assertEqual(errorno, None)
        pyuv.fs.close(self.loop, fd, self.close_cb)

    def test_open_create(self):
        self.errorno = None
        pyuv.fs.open(self.loop, TEST_FILE, os.O_WRONLY|os.O_CREAT, stat.S_IREAD|stat.S_IWRITE, self.open_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)

    def open_noent_cb(self, loop, path, fd, errorno):
        self.fd = fd
        self.errorno = errorno

    def test_open_noent(self):
        self.fd = None
        self.errorno = None
        pyuv.fs.open(self.loop, BAD_FILE, os.O_RDONLY, 0, self.open_noent_cb)
        self.loop.run()
        self.assertEqual(self.fd, None)
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)

    def test_open_sync(self):
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_WRONLY|os.O_CREAT|os.O_TRUNC, stat.S_IREAD|stat.S_IWRITE)
        pyuv.fs.close(self.loop, fd)

    def test_open_error_sync(self):
        try:
            pyuv.fs.open(self.loop, BAD_FILE, os.O_RDONLY, 0)
        except pyuv.error.FSError as e:
            self.errorno = e.args[0]
        else:
            self.errorno = None
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)


class FSTestRead(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write('test1234567890')

    def tearDown(self):
        os.remove(TEST_FILE)

    def read_cb(self, loop, path, read_data, errorno):
        self.errorno = errorno
        self.data = read_data
        pyuv.fs.close(self.loop, self.fd)

    def test_read(self):
        self.data = None
        self.errorno = None
        self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDONLY, stat.S_IREAD)
        pyuv.fs.read(self.loop, self.fd, 4, -1, self.read_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertEqual(self.data, b'test')

    def test_read_sync(self):
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDONLY, stat.S_IREAD)
        self.data = pyuv.fs.read(self.loop, fd, 4, -1)
        pyuv.fs.close(self.loop, fd)
        self.assertEqual(self.data, b'test')

    def test_read_offset(self):
        with open(TEST_FILE, 'w') as f:
            f.seek(MAX_INT32_VALUE)
            f.write('test1234567890')
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDONLY, stat.S_IREAD)
        data = pyuv.fs.read(self.loop, fd, 4, MAX_INT32_VALUE + 4)
        pyuv.fs.close(self.loop, fd)
        self.assertEqual(data, b'1234')


class FSTestWrite(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_WRONLY|os.O_CREAT|os.O_TRUNC, stat.S_IWRITE|stat.S_IREAD)

    def tearDown(self):
        os.remove(TEST_FILE)

    def write_cb(self, loop, path, bytes_written, errorno):
        pyuv.fs.close(self.loop, self.fd)
        self.bytes_written = bytes_written
        self.errorno = errorno

    def test_write(self):
        self.bytes_written = None
        self.errorno = None
        pyuv.fs.write(self.loop, self.fd, "TEST", -1, self.write_cb)
        self.loop.run()
        self.assertEqual(self.bytes_written, 4)
        self.assertEqual(self.errorno, None)
        with open(TEST_FILE, 'r') as fobj:
            self.assertEqual(fobj.read(), "TEST")

    def test_write_null(self):
        self.bytes_written = None
        self.errorno = None
        pyuv.fs.write(self.loop, self.fd, "TES\x00T", -1, self.write_cb)
        self.loop.run()
        self.assertEqual(self.bytes_written, 5)
        self.assertEqual(self.errorno, None)
        with open(TEST_FILE, 'r') as fobj:
            self.assertEqual(fobj.read(), "TES\x00T")

    def test_write_sync(self):
        self.bytes_written = pyuv.fs.write(self.loop, self.fd, "TEST", -1)
        pyuv.fs.close(self.loop, self.fd)
        self.assertEqual(self.bytes_written, 4)
        with open(TEST_FILE, 'r') as fobj:
            self.assertEqual(fobj.read(), "TEST")

    def test_write_offset(self):
        offset = MAX_INT32_VALUE + 4
        self.bytes_written = pyuv.fs.write(self.loop, self.fd, "TEST", offset)
        pyuv.fs.close(self.loop, self.fd)
        with open(TEST_FILE, 'r') as fobj:
            fobj.seek(offset)
            self.assertEqual(fobj.read(), "TEST")


class FSTestFsync(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()

    def write_cb(self, loop, path, bytes_written, errorno):
        self.assertEqual(bytes_written, 4)
        self.assertEqual(errorno, None)
        pyuv.fs.fdatasync(self.loop, self.fd, self.fdatasync_cb)

    def fdatasync_cb(self, loop, path, errorno):
        self.assertEqual(errorno, None)
        pyuv.fs.fsync(self.loop, self.fd, self.fsync_cb)

    def fsync_cb(self, loop, path, errorno):
        self.assertEqual(errorno, None)

    def test_fsync(self):
        self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR|os.O_CREAT|os.O_TRUNC, stat.S_IREAD|stat.S_IWRITE)
        pyuv.fs.write(self.loop, self.fd, "TEST", -1, self.write_cb)
        self.loop.run()
        pyuv.fs.close(self.loop, self.fd)
        with open(TEST_FILE, 'r') as fobj:
            self.assertEqual(fobj.read(), "TEST")

    def test_fsync_sync(self):
        self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR|os.O_CREAT|os.O_TRUNC, stat.S_IREAD|stat.S_IWRITE)
        pyuv.fs.write(self.loop, self.fd, "TEST", -1)
        pyuv.fs.fdatasync(self.loop, self.fd)
        pyuv.fs.fsync(self.loop, self.fd)
        pyuv.fs.close(self.loop, self.fd)
        with open(TEST_FILE, 'r') as fobj:
            self.assertEqual(fobj.read(), "TEST")


class FSTestFtruncate(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write("test-data")

    def tearDown(self):
        os.remove(TEST_FILE)

    def ftruncate_cb(self, loop, path, errorno):
        self.errorno = errorno
        pyuv.fs.close(self.loop, self.fd)

    def test_ftruncate1(self):
        self.errorno = None
        self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
        pyuv.fs.ftruncate(self.loop, self.fd, 4, self.ftruncate_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        with open(TEST_FILE, 'r') as fobj:
            self.assertEqual(fobj.read(), "test")

    def test_ftruncate2(self):
        self.errorno = None
        self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
        pyuv.fs.ftruncate(self.loop, self.fd, 0, self.ftruncate_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        with open(TEST_FILE, 'r') as fobj:
            self.assertEqual(fobj.read(), "")

    def test_ftruncate_sync(self):
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
        pyuv.fs.ftruncate(self.loop, fd, 0)
        pyuv.fs.close(self.loop, fd)
        with open(TEST_FILE, 'r') as fobj:
            self.assertEqual(fobj.read(), "")


class FSTestReaddir(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        os.mkdir(TEST_DIR, 0o755)
        os.mkdir(os.path.join(TEST_DIR, TEST_DIR2), 0o755)
        with open(os.path.join(TEST_DIR, TEST_FILE), 'w') as f:
            f.write('test')
        with open(os.path.join(TEST_DIR, TEST_FILE2), 'w') as f:
            f.write('test')

    def tearDown(self):
        shutil.rmtree(TEST_DIR)

    def readdir_cb(self, loop, path, files, errorno):
        self.errorno = errorno
        self.files = files

    def test_bad_readdir(self):
        self.errorno = None
        self.files = None
        pyuv.fs.readdir(self.loop, BAD_DIR, 0, self.readdir_cb)
        self.loop.run()
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)

    def test_readdir(self):
        self.errorno = None
        self.files = None
        pyuv.fs.readdir(self.loop, TEST_DIR, 0, self.readdir_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertTrue(TEST_FILE in self.files)
        self.assertTrue(TEST_FILE2 in self.files)
        self.assertTrue(TEST_DIR2 in self.files)

    def test_readdir_sync(self):
        self.files = pyuv.fs.readdir(self.loop, TEST_DIR, 0)
        self.assertNotEqual(self.files, None)
        self.assertTrue(TEST_FILE in self.files)
        self.assertTrue(TEST_FILE2 in self.files)
        self.assertTrue(TEST_DIR2 in self.files)

    def test_readdir_error_sync(self):
        try:
            pyuv.fs.readdir(self.loop, BAD_DIR, 0)
        except pyuv.error.FSError as e:
            self.errorno = e.args[0]
        else:
            self.errorno = None
        self.assertEqual(self.errorno, pyuv.errno.UV_ENOENT)


class FSTestSendfile(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write("begin\n")
            os.lseek(f.fileno(), 65536, os.SEEK_CUR)
            f.write("end\n")
            f.flush()

    def tearDown(self):
        os.remove(TEST_FILE)
        os.remove(TEST_FILE2)

    def sendfile_cb(self, loop, path, bytes_written, errorno):
        self.bytes_written = bytes_written
        self.errorno = errorno

    def test_sendfile(self):
        self.result = None
        self.errorno = None
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
        fd2 = pyuv.fs.open(self.loop, TEST_FILE2, os.O_RDWR|os.O_CREAT, stat.S_IREAD|stat.S_IWRITE)
        pyuv.fs.sendfile(self.loop, fd2, fd, 0, 131072, self.sendfile_cb)
        self.loop.run()
        pyuv.fs.close(self.loop, fd)
        pyuv.fs.close(self.loop, fd2)
        self.assertEqual(self.errorno, None)
        with open(TEST_FILE, 'r') as fobj1:
            with open(TEST_FILE2, 'r') as fobj2:
                self.assertEqual(fobj1.read(), fobj2.read())

    def test_sendfile_sync(self):
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
        fd2 = pyuv.fs.open(self.loop, TEST_FILE2, os.O_RDWR|os.O_CREAT, stat.S_IREAD|stat.S_IWRITE)
        self.bytes_written = pyuv.fs.sendfile(self.loop, fd2, fd, 0, 131072)
        pyuv.fs.close(self.loop, fd)
        pyuv.fs.close(self.loop, fd2)
        with open(TEST_FILE, 'r') as fobj1:
            with open(TEST_FILE2, 'r') as fobj2:
                self.assertEqual(fobj1.read(), fobj2.read())

    def test_sendfile_offset(self):
        offset = MAX_INT32_VALUE + 1
        with open(TEST_FILE, 'w') as f:
            f.seek(offset)
            f.write("test")
            f.flush()
        fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IREAD|stat.S_IWRITE)
        fd2 = pyuv.fs.open(self.loop, TEST_FILE2, os.O_RDWR|os.O_CREAT, stat.S_IREAD|stat.S_IWRITE)
        self.bytes_written = pyuv.fs.sendfile(self.loop, fd2, fd, offset, 4)
        pyuv.fs.close(self.loop, fd)
        pyuv.fs.close(self.loop, fd2)
        with open(TEST_FILE, 'r') as fobj1:
            fobj1.seek(offset)
            with open(TEST_FILE2, 'r') as fobj2:
                self.assertEqual(fobj1.read(), fobj2.read())


class FSTestUtime(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write("test")
        self.fd = None

    def tearDown(self):
        os.remove(TEST_FILE)

    def utime_cb(self, loop, path, errorno):
        self.errorno = errorno
        if self.fd is not None:
            pyuv.fs.close(self.loop, self.fd)

    def test_utime(self):
        self.errorno = None
        atime = mtime = 400497753
        pyuv.fs.utime(self.loop, TEST_FILE, atime, mtime, self.utime_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        s = os.stat(TEST_FILE)
        self.assertEqual(s.st_atime, atime)
        self.assertEqual(s.st_mtime, mtime)

    def test_futime(self):
        self.errorno = None
        atime = mtime = 400497753
        self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IWRITE|stat.S_IREAD)
        pyuv.fs.futime(self.loop, self.fd, atime, mtime, self.utime_cb)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        s = os.stat(TEST_FILE)
        self.assertTrue(s.st_atime == atime and s.st_mtime == mtime)

    def test_utime_sync(self):
        atime = mtime = 400497753
        pyuv.fs.utime(self.loop, TEST_FILE, atime, mtime)
        s = os.stat(TEST_FILE)
        self.assertEqual(s.st_atime, atime)
        self.assertEqual(s.st_mtime, mtime)

    def test_futime_sync(self):
        atime = mtime = 400497753
        self.fd = pyuv.fs.open(self.loop, TEST_FILE, os.O_RDWR, stat.S_IWRITE|stat.S_IREAD)
        pyuv.fs.futime(self.loop, self.fd, atime, mtime)
        pyuv.fs.close(self.loop, self.fd)
        s = os.stat(TEST_FILE)
        self.assertEqual(s.st_atime, atime)
        self.assertEqual(s.st_mtime, mtime)


class FSEventTestBasic(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        with open(TEST_FILE, 'w') as f:
            f.write("test")

    def tearDown(self):
        try:
            os.remove(TEST_FILE)
        except OSError:
            pass
        try:
            os.remove(TEST_FILE2)
        except OSError:
            pass

    def on_fsevent_cb(self, handle, filename, events, errorno):
        handle.close()
        self.filename = filename
        self.events = events
        self.errorno = errorno

    def timer_cb(self, timer):
        timer.close()
        os.rename(TEST_FILE, TEST_FILE2)

    def test_fsevent_basic(self):
        self.errorno = None
        self.events = None
        self.filename = None
        fs_event = pyuv.fs.FSEvent(self.loop, TEST_FILE, self.on_fsevent_cb, 0)
        timer = pyuv.Timer(self.loop)
        timer.start(self.timer_cb, 1, 0)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertTrue(self.filename == None or self.filename == TEST_FILE)
        self.assertTrue(self.events in (pyuv.fs.UV_CHANGE, pyuv.fs.UV_RENAME))


class FSEventTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        os.mkdir(TEST_DIR, 0o755)
        with open(os.path.join(TEST_DIR, TEST_FILE), 'w') as f:
            f.write("test")
        with open(TEST_FILE, 'w') as f:
            f.write("test")

    def tearDown(self):
        shutil.rmtree(TEST_DIR)
        try:
            os.remove(TEST_FILE)
        except OSError:
            pass
        try:
            os.remove(TEST_FILE2)
        except OSError:
            pass

    def on_fsevent_cb(self, handle, filename, events, errorno):
        handle.close()
        self.filename = filename
        self.events = events
        self.errorno = errorno

    def timer_cb2(self, timer):
        timer.close()
        os.rename(os.path.join(TEST_DIR, TEST_FILE), os.path.join(TEST_DIR, TEST_FILE2))

    def test_fsevent_dir(self):
        self.errorno = None
        self.events = None
        self.filename = None
        fs_event = pyuv.fs.FSEvent(self.loop, TEST_DIR, self.on_fsevent_cb, 0)
        timer = pyuv.Timer(self.loop)
        timer.start(self.timer_cb2, 1, 0)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertTrue(self.filename == None or self.filename == TEST_FILE)
        self.assertTrue(self.events & pyuv.fs.UV_RENAME)

    def timer_cb3(self, timer):
        timer.close()
        os.utime(os.path.join(TEST_DIR, TEST_FILE), None)

    def test_fsevent_nrefile(self):
        self.errorno = None
        self.events = None
        self.filename = None
        fs_event = pyuv.fs.FSEvent(self.loop, os.path.join(TEST_DIR, TEST_FILE), self.on_fsevent_cb, 0)
        timer = pyuv.Timer(self.loop)
        timer.start(self.timer_cb3, 1, 0)
        self.loop.run()
        self.assertEqual(self.errorno, None)
        self.assertTrue(self.filename == None or self.filename == TEST_FILE)
        self.assertTrue(self.events & pyuv.fs.UV_CHANGE)


class FSPollTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()

    def tearDown(self):
        try:
            os.remove(TEST_FILE)
        except OSError:
            pass

    def _touch_file(self):
        with open(TEST_FILE, 'w+') as f:
            self.count += 1
            for i in range(self.count+1):
                f.write('*')

    def on_timer(self, timer):
        self._touch_file()

    def on_fspoll(self, handle, prev_stat, curr_stat, error):
        if self.poll_cb_called == 0:
            self.assertEqual(error, pyuv.errno.UV_ENOENT)
            self._touch_file()
        elif self.poll_cb_called == 1:
            self.timer.start(self.on_timer, 0.02, 0.0)
        elif self.poll_cb_called == 2:
            self.timer.start(self.on_timer, 0.2, 0.0)
        elif self.poll_cb_called == 3:
            os.remove(TEST_FILE)
        elif self.poll_cb_called == 4:
            self.assertEqual(error, pyuv.errno.UV_ENOENT)
            self.fs_poll.close()
            self.timer.close()
        else:
            self.fail('This should not happen')
        self.poll_cb_called += 1

    def test_fspoll1(self):
        self.count = 0
        self.poll_cb_called = 0
        self.timer = pyuv.Timer(self.loop)
        self.fs_poll = pyuv.fs.FSPoll(self.loop)
        self.fs_poll.start(TEST_FILE, self.on_fspoll, 0.1)
        self.loop.run()
        self.assertEqual(self.poll_cb_called, 5)


if __name__ == '__main__':
    unittest2.main(verbosity=2)




#!/usr/bin/env python

import sys
sys.path.insert(0, '../')

import pyuv


def on_write2(handle, error):
    global channel, recv_handle
    recv_handle.close()
    channel.close()

def on_channel_read(handle, data, pending, error):
    global channel, loop, recv_handle
    assert pending in (pyuv.UV_NAMED_PIPE, pyuv.UV_UDP, pyuv.UV_TCP), "wrong handle type"
    if pending == pyuv.UV_NAMED_PIPE:
        recv_handle = pyuv.Pipe(loop)
    elif pending == pyuv.UV_TCP:
        recv_handle = pyuv.TCP(loop)
    elif pending == pyuv.UV_UDP:
        recv_handle = pyuv.UDP(loop)
    channel.accept(recv_handle)
    channel.write2(b".", recv_handle, on_write2)


loop = pyuv.Loop.default_loop()

recv_handle = None

channel = pyuv.Pipe(loop, True)
channel.open(sys.stdin.fileno())
channel.start_read2(on_channel_read)

loop.run()





from common import unittest2
import pyuv


class LoopRunTest(unittest2.TestCase):

    def test_run_once(self):
        self.cb_called = 0
        def prepare_cb(handle):
            handle.close()
            self.cb_called += 1
        loop = pyuv.Loop.default_loop()
        for i in range(500):
            prepare = pyuv.Prepare(loop)
            prepare.start(prepare_cb)
            loop.run(pyuv.UV_RUN_ONCE)
        self.assertEqual(self.cb_called, 500)

    def test_run_nowait(self):
        self.cb_called = 0
        def timer_cb(handle):
            handle.close()
            self.cb_called = 1
        loop = pyuv.Loop.default_loop()
        timer = pyuv.Timer(loop)
        timer.start(timer_cb, 10, 10)
        loop.run(pyuv.UV_RUN_NOWAIT)
        self.assertEqual(self.cb_called, 0)

    def test_stop(self):
        self.num_ticks = 10
        self.prepare_called = 0
        self.timer_called = 0
        def timer_cb(handle):
            self.timer_called += 1
            if self.timer_called == 1:
                self.loop.stop()
            elif self.timer_called == self.num_ticks:
                handle.close()
        def prepare_cb(handle):
            self.prepare_called += 1
            if self.prepare_called == self.num_ticks:
                handle.close()
        self.loop = pyuv.Loop.default_loop()
        timer = pyuv.Timer(self.loop)
        timer.start(timer_cb, 0.1, 0.1)
        prepare = pyuv.Prepare(self.loop)
        prepare.start(prepare_cb)
        self.loop.run(pyuv.UV_RUN_DEFAULT)
        self.assertEqual(self.timer_called, 1)
        self.assertTrue(self.prepare_called >= 2)
        self.loop.run(pyuv.UV_RUN_NOWAIT)
        self.assertTrue(self.prepare_called >= 3)
        self.loop.run(pyuv.UV_RUN_DEFAULT)
        self.assertEqual(self.timer_called, 10)
        self.assertEqual(self.prepare_called, 10)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





import gc
import weakref

from common import unittest2
import pyuv


class Foo(object): pass

class GCTest(unittest2.TestCase):

    def test_gc(self):
        loop = pyuv.Loop.default_loop()
        timer = pyuv.Timer(loop)

        w_timer = weakref.ref(timer)
        self.assertNotEqual(w_timer(), None)

        foo = Foo()
        foo.timer = timer
        timer.foo = foo

        timer = None
        foo = None

        gc.collect()
        self.assertEqual(w_timer(), None)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





from common import unittest2

import pyuv
import socket


class UtilTest(unittest2.TestCase):

    def test_hrtime(self):
        r = pyuv.util.hrtime()
        self.assertTrue(r)

    def test_freemem(self):
        r = pyuv.util.get_free_memory()
        self.assertTrue(r)

    def test_totalmem(self):
        r = pyuv.util.get_total_memory()
        self.assertTrue(r)

    def test_loadavg(self):
        r = pyuv.util.loadavg()
        self.assertTrue(r)

    def test_uptime(self):
        r = pyuv.util.uptime()
        self.assertTrue(r)

    def test_resident_set_memory(self):
        r = pyuv.util.resident_set_memory()
        self.assertTrue(r)

    def test_interface_addresses(self):
        r = pyuv.util.interface_addresses()
        self.assertTrue(r)

    def test_cpu_info(self):
        r = pyuv.util.cpu_info()
        self.assertTrue(r)

    def test_getaddrinfo(self):
        def getaddrinfo_cb(result, errorno):
            self.assertEqual(errorno, None)
        loop = pyuv.Loop.default_loop()
        pyuv.util.getaddrinfo(loop, getaddrinfo_cb, 'localhost', 80, socket.AF_INET)
        loop.run()


if __name__ == '__main__':
    unittest2.main(verbosity=2)





from common import unittest2, linesep

import errno
import pyuv
import socket
import sys


TEST_PORT = 1234

if sys.platform == "win32":
    NONBLOCKING = (errno.WSAEWOULDBLOCK,)
else:
    NONBLOCKING = (errno.EAGAIN, errno.EINPROGRESS, errno.EWOULDBLOCK)


class PollTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.poll = None
        self.server = None
        self.sock = None
        self.client_connection = None
        self.connecting = False

    def on_connection(self, server, error):
        self.assertEqual(error, None)
        client = pyuv.TCP(pyuv.Loop.default_loop())
        server.accept(client)
        self.client_connection = client
        client.start_read(self.on_client_connection_read)
        client.write(b"PING"+linesep)

    def on_client_connection_read(self, client, data, error):
        self.assertEqual(data, b"PONG"+linesep)
        self.poll.close()
        self.client_connection.close()
        self.server.close()

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def poll_cb(self, handle, events, error):
        if self.connecting:
            err = self.sock.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
            if not err:
                self.connecting = False
                self.poll.stop()
                self.poll.start(pyuv.UV_READABLE, self.poll_cb)
            elif err in NONBLOCKING:
                return
            else:
                self.poll.close()
                self.client_connection.close()
                self.server.close()
                self.fail("Error connecting socket: %d" % err)
            return
        if (events & pyuv.UV_READABLE):
            self.poll.stop()
            data = self.sock.recv(1024)
            self.assertEqual(data, b"PING"+linesep)
            self.poll.start(pyuv.UV_WRITABLE, self.poll_cb)
        elif (events & pyuv.UV_WRITABLE):
            self.poll.stop()
            self.sock.send(b"PONG"+linesep)

    def test_poll(self):
        self.server = pyuv.TCP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.listen(self.on_connection)
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setblocking(False)
        while True:
            r = self.sock.connect_ex(("127.0.0.1", TEST_PORT))
            if r and r != errno.EINTR:
                break
        if r not in NONBLOCKING:
            self.server.close()
            self.fail("Error connecting socket: %d" % r)
            return
        self.connecting = True
        self.poll = pyuv.Poll(self.loop, self.sock.fileno())
        self.assertEqual(self.sock.fileno(), self.poll.fileno())
        self.poll.start(pyuv.UV_WRITABLE, self.poll_cb)
        self.loop.run()
        self.assertTrue(self.poll.closed)
        self.assertEqual(self.poll.fileno(), -1)


if __name__ == '__main__':
    unittest2.main(verbosity=2)






from common import unittest2
import pyuv


class MultiHandleTest(unittest2.TestCase):

    def test_multihandle1(self):
        self.close_cb_called = 0
        self.prepare_cb_called = 0
        def close_cb(handle):
            self.close_cb_called += 1
        def prepare_cb(prepare):
            self.prepare_cb_called += 1
            prepare.stop()
            prepare.close(close_cb)
        self.idle_cb_called = 0
        def idle_cb(idle):
            self.idle_cb_called += 1
            idle.stop()
            idle.close(close_cb)
        self.check_cb_called = 0
        def check_cb(check):
            self.check_cb_called += 1
            check.stop()
            check.close(close_cb)
        self.timer_cb_called = 0
        def timer_cb(timer):
            self.timer_cb_called += 1
            timer.stop()
            timer.close(close_cb)
        loop = pyuv.Loop.default_loop()
        prepare = pyuv.Prepare(loop)
        prepare.start(prepare_cb)
        idle = pyuv.Idle(loop)
        idle.start(idle_cb)
        check = pyuv.Check(loop)
        check.start(check_cb)
        timer = pyuv.Timer(loop)
        timer.start(timer_cb, 0.1, 0)
        loop.run()
        self.assertEqual(self.prepare_cb_called, 1)
        self.assertEqual(self.idle_cb_called, 1)
        self.assertEqual(self.check_cb_called, 1)
        self.assertEqual(self.close_cb_called, 4)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





import gc
import weakref

from common import unittest2
import pyuv


class WalkTest(unittest2.TestCase):

    def test_walk(self):
        def timer_cb(timer):
            handles = []
            def walk_cb(handle):
                handles.append(handle)
            loop.walk(walk_cb)
            self.assertTrue(timer in handles)
            timer.close()
        loop = pyuv.Loop.default_loop()
        timer = pyuv.Timer(loop)
        w_timer = weakref.ref(timer)
        self.assertNotEqual(w_timer(), None)
        timer.start(timer_cb, 0.1, 0.0)
        loop.run()
        handles = []
        def walk_cb(handle):
            handles.append(handle)
        loop.walk(walk_cb)
        self.assertTrue(timer not in handles)
        handles = None
        timer = None
        gc.collect()
        self.assertEqual(w_timer(), None)



if __name__ == '__main__':
    unittest2.main(verbosity=2)





import functools
import sys
import threading
import time

from common import unittest2
import pyuv


class WorkItem(object):

    def __init__(self, func, *args, **kw):
        self._cb = functools.partial(func, *args, **kw)
        self.result = None
        self.exc_info = None

    def __call__(self):
        try:
            self._cb()
        except Exception:
            self.exc_info = sys.exc_info()
        finally:
            self._cb = None


class ThreadPoolTest(unittest2.TestCase):

    def setUp(self):
        self.pool_cb_called = 0
        self.pool_after_work_cb_called = 0
        self.loop = pyuv.Loop.default_loop()

    def run_in_pool(self, *args, **kw):
        self.pool_cb_called += 1
        time.sleep(0.1)

    def after_work_cb(self, error):
        self.assertEqual(error, None)
        self.pool_after_work_cb_called += 1

    def test_threadpool1(self):
        self.loop.queue_work(self.run_in_pool, self.after_work_cb)
        args = (1, 2, 3)
        kw = {}
        self.loop.queue_work(functools.partial(self.run_in_pool, *args, **kw), self.after_work_cb)
        args = ()
        kw = {'test': 1}
        self.loop.queue_work(functools.partial(self.run_in_pool, *args, **kw), self.after_work_cb)
        self.loop.run()
        self.assertEqual(self.pool_cb_called, 3)
        self.assertEqual(self.pool_after_work_cb_called, 3)

    def raise_in_pool(self, *args, **kw):
        1/0

    def after_work_cb2(self, error):
        self.assertEqual(error, None)
        self.assertEqual(self.work_item.exc_info[0], ZeroDivisionError)

    def test_threadpool_exc(self):
        self.work_item = WorkItem(self.raise_in_pool)
        self.loop.queue_work(self.work_item, self.after_work_cb2)
        self.loop.run()


class ThreadPoolMultiLoopTest(unittest2.TestCase):

    def setUp(self):
        self.pool_cb_called = 0
        self.lock = threading.Lock()

    def run_in_pool(self):
        with self.lock:
            self.pool_cb_called += 1
            time.sleep(0.2)

    def run_loop(self):
        loop = pyuv.Loop()
        loop.queue_work(self.run_in_pool)
        loop.run()

    def test_threadpool_multiple_loops(self):
        t1 = threading.Thread(target=self.run_loop)
        t2 = threading.Thread(target=self.run_loop)
        t3 = threading.Thread(target=self.run_loop)
        [t.start() for t in (t1, t2, t3)]
        [t.join(5) for t in (t1, t2, t3)]
        self.assertEqual(self.pool_cb_called, 3)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





from common import unittest2, platform_skip
import common
import pyuv
import socket
import sys


TEST_PORT = 12345
TEST_PORT2 = 12346
MULTICAST_ADDRESS = "239.255.0.1"

class UDPTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None

    def on_close(self, handle):
        self.on_close_called += 1

    def on_server_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        data = data.strip()
        self.assertEqual(data, b"PING")
        self.server.send((ip, port), b"PONG"+common.linesep)

    def on_client_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        data = data.strip()
        self.assertEqual(data, b"PONG")
        self.client.close(self.on_close)
        self.server.close(self.on_close)

    def timer_cb(self, timer):
        self.client.send(("127.0.0.1", TEST_PORT), b"PING"+common.linesep)
        timer.close(self.on_close)

    def test_udp_pingpong(self):
        self.on_close_called = 0
        self.server = pyuv.UDP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.set_broadcast(True) # for coverage
        try:
            self.server.set_ttl(10) # for coverage
        except pyuv.error.UDPError:
            # This function is not implemented on Windows
            pass
        self.server.start_recv(self.on_server_recv)
        self.client = pyuv.UDP(self.loop)
        self.client.bind(("0.0.0.0", TEST_PORT2))
        self.client.start_recv(self.on_client_recv)
        timer = pyuv.Timer(self.loop)
        timer.start(self.timer_cb, 0.1, 0)
        self.loop.run()
        self.assertEqual(self.on_close_called, 3)


class UDPTestNull(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None

    def on_close(self, handle):
        self.on_close_called += 1

    def on_server_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        data = data.strip()
        self.assertEqual(data, b"PIN\x00G")
        self.server.send((ip, port), b"PONG"+common.linesep)

    def on_client_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        data = data.strip()
        self.assertEqual(data, b"PONG")
        self.client.close(self.on_close)
        self.server.close(self.on_close)

    def timer_cb(self, timer):
        self.client.send(("127.0.0.1", TEST_PORT), b"PIN\x00G"+common.linesep)
        timer.close(self.on_close)

    def test_udp_pingpong_null(self):
        self.on_close_called = 0
        self.server = pyuv.UDP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.start_recv(self.on_server_recv)
        self.client = pyuv.UDP(self.loop)
        self.client.bind(("0.0.0.0", TEST_PORT2))
        self.client.start_recv(self.on_client_recv)
        timer = pyuv.Timer(self.loop)
        timer.start(self.timer_cb, 0.1, 0)
        self.loop.run()
        self.assertEqual(self.on_close_called, 3)


class UDPTestList(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None

    def on_close(self, handle):
        self.on_close_called += 1

    def on_server_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        data = data.strip()
        self.assertEqual(data, b"PING")
        self.server.sendlines((ip, port), [b"PONG", common.linesep])

    def on_client_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        data = data.strip()
        self.assertEqual(data, b"PONG")
        self.client.close(self.on_close)
        self.server.close(self.on_close)

    def timer_cb(self, timer):
        self.client.sendlines(("127.0.0.1", TEST_PORT), [b"PING", common.linesep])
        timer.close(self.on_close)

    def test_udp_pingpong_list(self):
        self.on_close_called = 0
        self.server = pyuv.UDP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.start_recv(self.on_server_recv)
        self.client = pyuv.UDP(self.loop)
        self.client.bind(("0.0.0.0", TEST_PORT2))
        self.client.start_recv(self.on_client_recv)
        timer = pyuv.Timer(self.loop)
        timer.start(self.timer_cb, 0.1, 0)
        self.loop.run()
        self.assertEqual(self.on_close_called, 3)


class UDPTestListNull(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None

    def on_close(self, handle):
        self.on_close_called += 1

    def on_server_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        data = data.strip()
        self.assertEqual(data, b"PIN\x00G")
        self.server.sendlines((ip, port), [b"PONG", common.linesep])

    def on_client_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        data = data.strip()
        self.assertEqual(data, b"PONG")
        self.client.close(self.on_close)
        self.server.close(self.on_close)

    def timer_cb(self, timer):
        self.client.sendlines(("127.0.0.1", TEST_PORT), [b"PIN\x00G", common.linesep])
        timer.close(self.on_close)

    def test_udp_pingpong_list_null(self):
        self.on_close_called = 0
        self.server = pyuv.UDP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.start_recv(self.on_server_recv)
        self.client = pyuv.UDP(self.loop)
        self.client.bind(("0.0.0.0", TEST_PORT2))
        self.client.start_recv(self.on_client_recv)
        timer = pyuv.Timer(self.loop)
        timer.start(self.timer_cb, 0.1, 0)
        self.loop.run()
        self.assertEqual(self.on_close_called, 3)


class UDPTestInvalidData(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None

    def on_close(self, handle):
        self.on_close_called += 1

    def on_server_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        self.client.close(self.on_close)
        self.server.close(self.on_close)
        self.fail("Expected send to fail.")

    def timer_cb(self, timer):
        self.assertRaises(TypeError, self.client.send, ("127.0.0.1", TEST_PORT), object())
        self.assertRaises(TypeError, self.client.send, ("127.0.0.1", TEST_PORT), 1)
        self.assertRaises(TypeError, self.client.sendlines, ("127.0.0.1", TEST_PORT), object())
        self.assertRaises(TypeError, self.client.sendlines, ("127.0.0.1", TEST_PORT), 1)

        self.client.close(self.on_close)
        self.server.close(self.on_close)
        timer.close(self.on_close)

    def test_udp_invalid_data(self):
        self.on_close_called = 0
        self.server = pyuv.UDP(self.loop)
        self.server.bind(("0.0.0.0", TEST_PORT))
        self.server.start_recv(self.on_server_recv)
        self.client = pyuv.UDP(self.loop)
        self.client.bind(("0.0.0.0", TEST_PORT2))
        timer = pyuv.Timer(self.loop)
        timer.start(self.timer_cb, 0.1, 0)
        self.loop.run()
        self.assertEqual(self.on_close_called, 3)


if sys.platform == 'win32':
    def interface_addresses():
        for family, _, _, _, sockaddr in socket.getaddrinfo('', None):
            if family == socket.AF_INET:
                yield sockaddr[0]
else:
    def interface_addresses():
        yield MULTICAST_ADDRESS


class UDPTestMulticast(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.clients = None
        self.received_data = None

    def on_close(self, handle):
        self.on_close_called += 1

    def on_client_recv(self, handle, ip_port, flags, data, error):
        self.assertEqual(flags, 0)
        ip, port = ip_port
        self.received_data = data.strip()

        for client in self.clients:
            client.set_membership(MULTICAST_ADDRESS, pyuv.UV_LEAVE_GROUP)
            client.close(self.on_close)

    def on_server_send(self, handle, error):
        handle.close(self.on_close)

    def _create_clients(self, loop=False):
        self.clients = list()
        for addr in interface_addresses():
            client = pyuv.UDP(self.loop)
            client.bind((addr, TEST_PORT))
            client.set_membership(MULTICAST_ADDRESS, pyuv.UV_JOIN_GROUP)
            if loop:
                client.set_multicast_loop(True)
            else:
                client.set_multicast_ttl(10)
            client.start_recv(self.on_client_recv)
            self.clients.append(client)

    def test_udp_multicast(self):
        self.on_close_called = 0
        self.server = pyuv.UDP(self.loop)
        self._create_clients()
        self.server.send((MULTICAST_ADDRESS, TEST_PORT), b"PING", self.on_server_send)
        self.loop.run()
        self.assertEqual(self.on_close_called, 1 + len(self.clients))
        self.assertEqual(self.received_data, b"PING")

    @platform_skip(["darwin"])
    def test_udp_multicast_loop(self):
        self.on_close_called = 0
        self._create_clients(True)
        for client in self.clients:
            client.send((MULTICAST_ADDRESS, TEST_PORT), b"PING")
        self.loop.run()
        self.assertEqual(self.on_close_called, len(self.clients))
        self.assertEqual(self.received_data, b"PING")


class UDPTestBigDatagram(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()

    def send_cb(self, handle, error):
        self.handle.close()
        self.errorno = error

    def test_udp_big_datagram(self):
        self.errorno = None
        self.handle = pyuv.UDP(self.loop)
        data = b"X"*65536
        self.handle.send(("127.0.0.1", TEST_PORT), data, self.send_cb)
        self.loop.run()
        self.assertEqual(self.errorno, pyuv.errno.UV_EMSGSIZE)


class UDPTestOpen(unittest2.TestCase):

    def test_udp_open(self):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        loop = pyuv.Loop.default_loop()
        handle = pyuv.UDP(loop)
        handle.open(sock.fileno())
        try:
            handle.bind(("1.2.3.4", TEST_PORT))
        except pyuv.error.UDPError as e:
            self.assertEqual(e.args[0], pyuv.errno.UV_EADDRNOTAVAIL)
        loop.run()
        sock.close()


if __name__ == '__main__':
    unittest2.main(verbosity=2)





from __future__ import print_function

import signal
import sys
import pyuv


def on_tty_read(handle, data, error):
    if data is None or data == b"exit":
        tty_stdin.close()
        tty_stdout.close()
    else:
        tty_stdout.write(data)

def signal_cb(handle, signum):
    tty_stdin.close()
    tty_stdout.close()
    signal_h.close()


loop = pyuv.Loop.default_loop()

tty_stdin = pyuv.TTY(loop, sys.stdin.fileno(), True)
tty_stdin.start_read(on_tty_read)
tty_stdout = pyuv.TTY(loop, sys.stdout.fileno(), False)

if sys.platform != "win32":
    print("Window size: (%d, %d)" % tty_stdin.get_winsize())

signal_h = pyuv.Signal(loop)
signal_h.start(signal_cb, signal.SIGINT)

loop.run()

pyuv.TTY.reset_mode()




# coding=utf-8

try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension
from setup_libuv import libuv_build_ext, libuv_sdist


__version__ = "0.10.12"

setup(name             = "pyuv",
      version          = __version__,
      author           = "Saúl Ibarra Corretgé",
      author_email     = "saghul@gmail.com",
      url              = "http://github.com/saghul/pyuv",
      description      = "Python interface for libuv",
      long_description = open("README.rst").read(),
      platforms        = ["POSIX", "Microsoft Windows"],
      classifiers      = [
          "Development Status :: 4 - Beta",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: MIT License",
          "Operating System :: POSIX",
          "Operating System :: Microsoft :: Windows",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.0",
          "Programming Language :: Python :: 3.1",
          "Programming Language :: Python :: 3.2",
          "Programming Language :: Python :: 3.3",
          "Programming Language :: Python :: 3.4"
      ],
      cmdclass     = {'build_ext': libuv_build_ext,
                      'sdist'    : libuv_sdist},
      ext_modules  = [Extension('pyuv',
                                sources = ['src/pyuv.c'],
                                define_macros=[('MODULE_VERSION', __version__),
                                               ('LIBUV_REVISION', libuv_build_ext.libuv_revision)]
                     )]
     )





import sys

from common import unittest2, platform_skip
import pyuv


@platform_skip(["win32"])
class TYTest(unittest2.TestCase):

    def test_tty1(self):
        loop = pyuv.Loop.default_loop()
        tty = pyuv.TTY(loop, sys.stdin.fileno(), True)
        w, h = tty.get_winsize()
        self.assertNotEqual((w, h), (None, None))

        tty.close()

        loop.run()
        tty.reset_mode()


if __name__ == '__main__':
    unittest2.main(verbosity=2)





import threading

from common import unittest2
import pyuv


class AsyncTest(unittest2.TestCase):

    def test_async1(self):
        self.async_cb_called = 0
        self.prepare_cb_called = 0
        def async_cb(async):
            with self.lock:
                self.async_cb_called += 1
                n = self.async_cb_called
            if n == 3:
                self.async.close()
                self.prepare.close()
        def prepare_cb(prepare):
            if self.prepare_cb_called:
                return
            self.prepare_cb_called += 1
            self.thread = threading.Thread(target=thread_cb)
            self.thread.start()
        def thread_cb():
            while True:
                with self.lock:
                    n = self.async_cb_called
                    if n == 3:
                        break
                    self.async.send()
        self.loop = pyuv.Loop.default_loop()
        self.async = pyuv.Async(self.loop, async_cb)
        self.prepare = pyuv.Prepare(self.loop)
        self.prepare.start(prepare_cb)
        self.lock = threading.Lock()
        self.loop.run()
        self.assertEqual(self.async_cb_called, 3)
        self.assertEqual(self.prepare_cb_called, 1)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





import sys
import socket
import signal
import weakref
import errno
import logging
import pyuv

logging.basicConfig(level=logging.DEBUG)

STOPSIGNALS = (signal.SIGINT, signal.SIGTERM)
NONBLOCKING = (errno.EAGAIN, errno.EWOULDBLOCK)
if sys.platform == "win32":
    NONBLOCKING = NONBLOCKING + (errno.WSAEWOULDBLOCK,)


class Connection(object):

    def __init__(self, sock, address, loop):
        self.sock = sock
        self.address = address
        self.sock.setblocking(0)
        self.buf = ""
        self.watcher = pyuv.Poll(loop, self.sock._sock)
        self.watcher.start(pyuv.UV_READABLE, self.io_cb)
        logging.debug("{0}: ready".format(self))

    def reset(self, events):
        self.watcher.start(events, self.io_cb)

    def handle_error(self, msg, level=logging.ERROR, exc_info=True):
        logging.log(level, "{0}: {1} --> closing".format(self, msg), exc_info=exc_info)
        self.close()

    def handle_read(self):
        try:
            buf = self.sock.recv(1024)
        except socket.error as err:
            if err.args[0] not in NONBLOCKING:
                self.handle_error("error reading from {0}".format(self.sock))
        if buf:
            self.buf += buf
            self.reset(pyuv.UV_READABLE | pyuv.UV_WRITABLE)
        else:
            self.handle_error("connection closed by peer", logging.DEBUG, False)

    def handle_write(self):
        try:
            sent = self.sock.send(self.buf)
        except socket.error as err:
            if err.args[0] not in NONBLOCKING:
                self.handle_error("error writing to {0}".format(self.sock))
        else :
            self.buf = self.buf[sent:]
            if not self.buf:
                self.reset(pyuv.UV_READABLE)

    def io_cb(self, watcher, revents, error):
        if error is not None:
            logging.error("Error in connection: %d: %s" % (error, pyuv.errno.strerror(error)))
            return
        if revents & pyuv.UV_READABLE:
            self.handle_read()
        elif revents & pyuv.UV_WRITABLE:
            self.handle_write()

    def close(self):
        self.watcher.stop()
        self.watcher = None
        self.sock.close()
        logging.debug("{0}: closed".format(self))


class Server(object):

    def __init__(self, address):
        self.sock = socket.socket()
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.sock.bind(address)
        self.sock.setblocking(0)
        self.address = self.sock.getsockname()
        self.loop = pyuv.Loop.default_loop()
        self.poll_watcher = pyuv.Poll(self.loop, self.sock.fileno())
        self.async = pyuv.Async(self.loop, self.async_cb)
        self.conns = weakref.WeakValueDictionary()
        self.signal_watchers = set()

    def handle_error(self, msg, level=logging.ERROR, exc_info=True):
        logging.log(level, "{0}: {1} --> stopping".format(self, msg), exc_info=exc_info)
        self.stop()

    def signal_cb(self, handle, signum):
        self.async.send()

    def async_cb(self, handle):
        handle.close()
        self.stop()

    def io_cb(self, watcher, revents, error):
        try:
            while True:
                try:
                    sock, address = self.sock.accept()
                except socket.error as err:
                    if err.args[0] in NONBLOCKING:
                        break
                    else:
                        raise
                else:
                    self.conns[address] = Connection(sock, address, self.loop)
        except Exception:
            self.handle_error("error accepting a connection")

    def start(self):
        self.sock.listen(socket.SOMAXCONN)
        self.poll_watcher.start(pyuv.UV_READABLE, self.io_cb)
        for sig in STOPSIGNALS:
            handle = pyuv.Signal(self.loop)
            handle.start(self.signal_cb, sig)
            self.signal_watchers.add(handle)
        logging.debug("{0}: started on {0.address}".format(self))
        self.loop.run()
        logging.debug("{0}: stopped".format(self))

    def stop(self):
        self.poll_watcher.stop()
        for watcher in self.signal_watchers:
            watcher.stop()
        self.signal_watchers.clear()
        self.sock.close()
        for conn in self.conns.values():
            conn.close()
        logging.debug("{0}: stopping".format(self))


if __name__ == "__main__":
    server = Server(("127.0.0.1", 9876))
    server.start()





from common import unittest2
import pyuv


class CheckTest(unittest2.TestCase):

    def test_check1(self):
        self.check_cb_called = 0
        def check_cb(check):
            self.check_cb_called += 1
            check.stop()
            check.close()
        self.timer_cb_called = 0
        def timer_cb(timer):
            self.timer_cb_called += 1
            timer.stop()
            timer.close()
        loop = pyuv.Loop.default_loop()
        check = pyuv.Check(loop)
        check.start(check_cb)
        timer = pyuv.Timer(loop)
        timer.start(timer_cb, 0.1, 0)
        loop.run()
        self.assertEqual(self.check_cb_called, 1)


if __name__ == '__main__':
    unittest2.main(verbosity=2)





import sys

from common import unittest2, platform_skip
import common
import pyuv

if sys.platform == 'win32':
    TEST_PIPE = '\\\\.\\pipe\\test-pipe'
else:
    TEST_PIPE = 'test-pipe'

BAD_PIPE = '/pipe/that/does/not/exist'


class PipeErrorTest(unittest2.TestCase):

    def on_client_connect_error(self, client_pipe, error):
        self.assertNotEqual(error, None)
        client_pipe.close()

    def test_client1(self):
        loop = pyuv.Loop.default_loop()
        client = pyuv.Pipe(loop)
        client.connect(BAD_PIPE, self.on_client_connect_error)
        loop.run()


class PipeTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        client = pyuv.Pipe(self.loop)
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.write(b"PING"+common.linesep)

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close()
            self.client_connections.remove(client)
            self.server.close()
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        data = data.strip()
        self.assertEqual(data, b"PING")
        client.close()

    def test_pipe1(self):
        self.server = pyuv.Pipe(self.loop)
        self.server.pending_instances(100)
        self.server.bind(TEST_PIPE)
        self.server.listen(self.on_connection)
        self.client = pyuv.Pipe(self.loop)
        self.client.connect(TEST_PIPE, self.on_client_connection)
        self.loop.run()


class PipeTestNull(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        client = pyuv.Pipe(self.loop)
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.write(b"PIN\x00G"+common.linesep)

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close()
            self.client_connections.remove(client)
            self.server.close()
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        data = data.strip()
        self.assertEqual(data, b"PIN\x00G")
        client.close()

    def test_pipe_null(self):
        self.server = pyuv.Pipe(self.loop)
        self.server.pending_instances(100)
        self.server.bind(TEST_PIPE)
        self.server.listen(self.on_connection)
        self.client = pyuv.Pipe(self.loop)
        self.client.connect(TEST_PIPE, self.on_client_connection)
        self.loop.run()


@platform_skip(["win32"])
class PipeTestList(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        client = pyuv.Pipe(self.loop)
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.writelines([b"PING", common.linesep])

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close()
            self.client_connections.remove(client)
            self.server.close()
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        self.assertEqual(data, b"PING"+common.linesep)
        client.close()

    def test_pipe_list(self):
        self.server = pyuv.Pipe(self.loop)
        self.server.pending_instances(100)
        self.server.bind(TEST_PIPE)
        self.server.listen(self.on_connection)
        self.client = pyuv.Pipe(self.loop)
        self.client.connect(TEST_PIPE, self.on_client_connection)
        self.loop.run()


@platform_skip(["win32"])
class PipeTestListNull(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        client = pyuv.Pipe(self.loop)
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.writelines([b"PING", b"\x00", common.linesep])

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close()
            self.client_connections.remove(client)
            self.server.close()
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        self.assertEqual(data, b"PING\x00"+common.linesep)
        client.close()

    def test_pipe_list_null(self):
        self.server = pyuv.Pipe(self.loop)
        self.server.pending_instances(100)
        self.server.bind(TEST_PIPE)
        self.server.listen(self.on_connection)
        self.client = pyuv.Pipe(self.loop)
        self.client.connect(TEST_PIPE, self.on_client_connection)
        self.loop.run()


class PipeShutdownTest(unittest2.TestCase):

    def setUp(self):
        self.loop = pyuv.Loop.default_loop()
        self.server = None
        self.client = None
        self.client_connections = []

    def on_connection(self, server, error):
        client = pyuv.Pipe(self.loop)
        server.accept(client)
        self.client_connections.append(client)
        client.start_read(self.on_client_connection_read)
        client.write(b"PING"+common.linesep)

    def on_client_connection_read(self, client, data, error):
        if data is None:
            client.close(self.on_close)
            self.client_connections.remove(client)
            self.server.close(self.on_close)
            return

    def on_client_connection(self, client, error):
        self.assertEqual(error, None)
        client.start_read(self.on_client_read)

    def on_close(self, handle):
        self.close_cb_called += 1

    def on_client_shutdown(self, client, error):
        self.shutdown_cb_called += 1
        client.close(self.on_close)

    def on_client_read(self, client, data, error):
        self.assertNotEqual(data, None)
        data = data.strip()
        self.assertEqual(data, b"PING")
        client.shutdown(self.on_client_shutdown)

    def test_pipe1(self):
        self.shutdown_cb_called = 0
        self.close_cb_called = 0
        self.server = pyuv.Pipe(self.loop)
        self.server.bind(TEST_PIPE)
        self.server.listen(self.on_connection)
        self.client = pyuv.Pipe(self.loop)
        self.client.connect(TEST_PIPE, self.on_client_connection)
        self.loop.run()
        self.assertEqual(self.shutdown_cb_called, 1)
        self.assertEqual(self.close_cb_called, 3)


if __name__ == '__main__':
    unittest2.main(verbosity=2)

